@@ -413,7 +413,7 @@ on '.' in attachments."
 					 nil nil nil nil)))
 	(mew-encode-syntax-print mew-encode-syntax)
 	;;
-	(find-file filepath)
+	(switch-to-buffer (mew-find-file-noselect2 filepath))
 	(text-mode)
 	;; buffer switched
 	(local-set-key "\C-c\C-q" 'mew-kill-buffer)))))
@@ -699,7 +699,7 @@ Input decryptors' addresses."
 	   efile FILE)
       (setq efile (if (string= subdir "") file (concat subdir file)))
       (setq FILE (expand-file-name efile attachdir))
-      (find-file FILE)
+      (switch-to-buffer (mew-find-file-noselect2 FILE))
       ;; buffer switched
       (local-set-key "\C-c\C-q" 'mew-kill-buffer)
       (if (file-symlink-p FILE)
