@@ -533,9 +533,10 @@ the name exists."
       (mew-complete-window-show all)
       (if (and (mew-characterp EXPAND-CHAR) (funcall fhit cmp ALIST))
 	  (message
-	   (substitute-command-keys
-	    "To expand '%s', type '%c' then '\\<mew-draft-header-map>\\[mew-draft-header-comp]'.")
-	   cmp EXPAND-CHAR)))
+	   "To expand '%s', type '%c' then '%s'"
+	   cmp EXPAND-CHAR
+ 	   (substitute-command-keys
+	    "\\<mew-draft-header-map>\\[mew-draft-header-comp]"))))
      ;; no candidate
      (t
       (insert WORD)
@@ -689,7 +690,7 @@ the name exists."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2002 Mew developing team.
+;; Copyright (C) 1997-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
