@@ -18,7 +18,7 @@
 ;;
 ;;
 
-(defun mew-cfent-value (case key def)
+(defun mew-cfent-value (case key def &optional symbol)
   (let ((cases (and case (mapcar 'mew-chop (mew-split case ?,))))
 	ent ret)
     (or (member mew-case-default cases)
@@ -27,8 +27,10 @@
     (catch 'loop
       (while cases
 	(setq ent (assoc key (mew-cfent-by-case (car cases))))
-	(setq ret (cdr ent))
-	(if ent (throw 'loop nil))
+	(when ent
+	  (setq ret (cdr ent))
+	  (if symbol (setq ret (symbol-value ret)))
+	  (if ent (throw 'loop nil)))
 	(setq cases (cdr cases)))
       (setq ret def))
     ret))
@@ -158,14 +160,14 @@
 (defun mew-smtp-mail-from (&optional case)
   (mew-cfent-value case "smtp-mail-from" mew-smtp-mail-from))
 
-(defun mew-smtp-msgid-domain (&optional case)
-  (or (mew-cfent-value case "smtp-msgid-domain" mew-smtp-msgid-domain)
-      (mew-mail-domain case)))
-
 (defun mew-smtp-msgid-user (&optional case)
   (or (mew-cfent-value case "smtp-msgid-user" mew-smtp-msgid-user)
       (mew-user case)))
 
+(defun mew-smtp-msgid-domain (&optional case)
+  (or (mew-cfent-value case "smtp-msgid-domain" mew-smtp-msgid-domain)
+      (mew-mail-domain case)))
+
 (defun mew-smtp-message-id (&optional case)
   (let* ((random (format "%08d" (mew-random)))
 	 (domain (mew-smtp-msgid-domain case))
@@ -285,6 +287,20 @@
 (defun mew-nntp-header-only (&optional case)
   (mew-cfent-value case "nntp-header-only" mew-nntp-header-only))
 
+(defun mew-nntp-msgid-user (&optional case)
+  (or (mew-cfent-value case "nntp-msgid-user" mew-nntp-msgid-user)
+      (mew-user case)))
+
+(defun mew-nntp-msgid-domain (&optional case)
+  (or (mew-cfent-value case "nntp-msgid-domain" mew-nntp-msgid-domain)
+      (mew-mail-domain case)))
+
+(defun mew-nntp-message-id (&optional case)
+  (let* ((random (format "%08d" (mew-random)))
+	 (domain (mew-nntp-msgid-domain case))
+	 (user (mew-nntp-msgid-user case))
+	 (time (mew-time-ctz-to-msgid (current-time))))
+    (concat "<" time "." random "." user "@" domain ">")))
 ;;
 
 (defun mew-inbox-folder (&optional case)
@@ -317,19 +333,67 @@
 (defun mew-content-type (&optional case)
   (mew-capitalize (mew-cfent-value case "content-type" mew-content-type)))
 
+(defun mew-use-retrieve (&optional case)
+  (mew-cfent-value case "use-retrieve" mew-use-retrieve))
+
+(defun mew-imap-spam-field (&optional case)
+  (mew-cfent-value case "imap-spam-field" mew-imap-spam-field))
+
+(defun mew-imap-spam-word (&optional case)
+  (mew-cfent-value case "imap-spam-word" mew-imap-spam-word))
+
 ;;
 
 (defun mew-proto (&optional case)
   (mew-cfent-value case "proto" mew-proto))
 
+;;
+
+(defun mew-ssh-prog (&optional case)
+  (mew-cfent-value case "ssh-prog" mew-ssh-prog))
+
+(defun mew-ssh-prog-args (&optional case)
+  (mew-cfent-value case "ssh-prog-args" mew-ssh-prog-args))
+
+(defun mew-ssh-prog-ver (&optional case)
+  (mew-cfent-value case "ssh-prog-ver" mew-ssh-prog-ver))
+
+(defun mew-spam-prog (&optional case)
+  (mew-cfent-value case "spam-prog" mew-spam-prog))
+
+(defun mew-spam-prog-args (&optional case)
+  (mew-cfent-value case "spam-prog-args" mew-spam-prog-args))
+
+(defun mew-ham-prog (&optional case)
+  (mew-cfent-value case "ham-prog" mew-ham-prog))
+
+(defun mew-ham-prog-args (&optional case)
+  (mew-cfent-value case "ham-prog-args" mew-ham-prog-args))
+
+;;
+
+(defun mew-refile-guess-alist (&optional case)
+  (mew-cfent-value case "refile-guess-alist" mew-refile-guess-alist 'symbol))
+
 ;;;
 ;;; Setting case
 ;;;
 
-(defvar mew-case-input nil)
-(defvar mew-case-output nil)
+(defvar mew-case-input nil
+  "\\<mew-summary-mode-map>This is used for
+1. '\\[mew-summary-retrieve]' to select a POP server (or something)
+2. Biff to select a server (or somthing)
+3. If 'mew-use-case-input-completion' is non-nil, this value 
+   is automatically inserted when inputing a folder>")
 
-(defvar mew-case-synchronize t)
+(defvar mew-case-output nil
+  "This is used to
+1. a draft. 
+   It is used to complete Newsgroups, 
+                 choose default Content-Type: for a file to be attached,
+                 choose a signature file, 
+                 select a SMTP server (or something).
+2. flushing a message queue.")
 
 (defun mew-case-set-both ()
   "Set the case both for input and output."
@@ -459,7 +523,7 @@ fields in the header according to the new value."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1999-2002 Mew developing team.
+;; Copyright (C) 1999-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
