@@ -100,7 +100,7 @@ Execute mew-dinfo-set before calling this."
   (let* ((buf (current-buffer))
 	 (case (mew-tinfo-get-case))
 	 (pnm (mew-smtp-info-name case))
-	 sendit resentp fcc err)
+	 sendit resentp fcc msg err)
     (if (get-process pnm)
 	(message "Another message is being sent. Try later")
       (goto-char (point-min))
@@ -122,18 +122,19 @@ Execute mew-dinfo-set before calling this."
 	      (narrow-to-region (point) (point))
 	      (mew-frwlet
 	       mew-cs-text-for-read mew-cs-text-for-write
-	       (insert-file-contents (mew-tinfo-get-hdr-file)))
+	       (mew-insert-file-contents (mew-tinfo-get-hdr-file)))
 	      (goto-char (point-min))
 	      (mew-header-goto-end)
 	      (mew-header-set mew-header-separator)
 	      (mew-header-delete-lines mew-field-delete-common)
 	      (mew-header-delete-lines (mew-tinfo-get-field-del)))
 	    (if (mew-smtp-encode pnm case resentp fcc nil nil 'header)
-		(cond
-		 ((eq action 'queue)
-		  (mew-smtp-queue case "from Draft mode"))
-		 ((eq action 'send)
-		  (mew-smtp-send-message case)))
+		(let ((mdi (concat (buffer-file-name) mew-draft-info-suffix)))
+		  (mew-delete-file mdi)
+		  (setq msg (mew-smtp-queue case "from Draft mode"))
+		  (if (eq action 'send)
+		      (mew-smtp-send-message 
+		       case (mew-queue-folder case) (list msg))))
 	      (setq err t))))
 	(when err
 	  (mew-current-set-window-config)
@@ -144,7 +145,7 @@ Execute mew-dinfo-set before calling this."
   (let* ((buf (current-buffer))
 	 (case (mew-tinfo-get-case))
 	 (pnm (mew-nntp2-info-name case))
-	 sendit resentp fcc err)
+	 sendit resentp fcc msg err)
     (if (get-process pnm)
 	(message "Another message is being posted. Try later")
       (goto-char (point-min))
@@ -166,18 +167,19 @@ Execute mew-dinfo-set before calling this."
 	      (narrow-to-region (point) (point))
 	      (mew-frwlet
 	       mew-cs-text-for-read mew-cs-text-for-write
-	       (insert-file-contents (mew-tinfo-get-hdr-file)))
+	       (mew-insert-file-contents (mew-tinfo-get-hdr-file)))
 	      (goto-char (point-min))
 	      (mew-header-goto-end)
 	      (mew-header-set mew-header-separator)
 	      (mew-header-delete-lines mew-field-delete-common)
 	      (mew-header-delete-lines (mew-tinfo-get-field-del)))
 	    (if (mew-nntp2-encode pnm case fcc nil nil 'header)
-		(cond
-		 ((eq action 'queue)
-		  (mew-nntp2-queue case "from Draft mode"))
-		 ((eq action 'send)
-		  (mew-nntp2-send-message case)))
+		(let ((mdi (concat (buffer-file-name) mew-draft-info-suffix)))
+		  (mew-delete-file mdi)
+		  (setq msg (mew-nntp2-queue case "from Draft mode"))
+		  (if (eq action 'send)
+		      (mew-nntp2-send-message 
+		       case (mew-postq-folder case) (list msg))))
 	      (setq err t))))
 	(when err
 	  (mew-current-set-window-config)
@@ -265,6 +267,8 @@ a text object to be saved can be specified."
 	   (setq end (mew-syntax-get-end (mew-syntax-get-part syntax)))
 	 (setq end (mew-syntax-get-end syntax)))
        (setq ctl (mew-syntax-get-ct syntax))
+       (setq cdpl (mew-syntax-get-cdp syntax))
+       (setq file (mew-syntax-get-filename cdpl ctl))
        ;;
        (if (and mew-use-samba-encoding
 		file
@@ -332,6 +336,7 @@ If executed with '\\[universal-argument]', coding-system is asked."
       (message "This command cannot be used on non-Mule Emacs")
     (mew-summary-msg-or-part
      ;; need to make a cache or a message buffer.
+     (mew-summary-toggle-disp-msg 'on)
      (mew-summary-display nil)
      (let* ((fld (mew-summary-folder-name))
 	    (msg (mew-summary-message-number2))
@@ -503,8 +508,9 @@ If called with '\\[universal-argument]', the current part is de-capsulated."
        (setq ret (mew-summary-burst-body fld msg folder num))
        (when ret
 	 (message "Bursting...done")
-	 (if (y-or-n-p (format "Go to %s? " folder))
-	     (mew-summary-visit-folder folder 'goend))
+	 (when mew-ask-folder-after-burst
+	   (if (y-or-n-p (format "Go to %s? " folder))
+	       (mew-summary-visit-folder folder 'goend)))
 	 (message "Messages from %s to %s were extracted in %s"
 		  (nth 0 ret) (nth 1 ret) folder))))))
 
@@ -521,8 +527,9 @@ If called with '\\[universal-argument]', the current part is de-capsulated."
 	  (car (car targets)) (cdr (car targets)) folder)
 	 (setq targets (cdr targets)))
        (message "Bursting...done")
-       (if (y-or-n-p (format "Go to %s? " folder))
-	   (mew-summary-visit-folder folder 'goend))))))
+       (when mew-ask-folder-after-burst
+	 (if (y-or-n-p (format "Go to %s? " folder))
+	     (mew-summary-visit-folder folder 'goend)))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -568,8 +575,9 @@ If called with '\\[universal-argument]', the current part is de-capsulated."
 	 (message "Bursting...done")
 	 (mew-touch-folder folder)
 	 (setq done t)))
-     (if (and done (y-or-n-p (format "Go to %s? " folder)))
-	 (mew-summary-visit-folder folder 'goend)))))
+     (when mew-ask-folder-after-burst
+       (if (and done (y-or-n-p (format "Go to %s? " folder)))
+	   (mew-summary-visit-folder folder 'goend))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
