@@ -183,8 +183,6 @@ DELS represents the messages to be deleted."
     (unless (file-directory-p trashdir) (make-directory trashdir))
     (setq num (string-to-int (mew-folder-new-message trash 'num-only cache)))
     ;; must be here after ensuring that +trash exists.
-
-
     (while dels
       (setq srcfile (mew-expand-folder src (car dels))) ;; xxx
       (setq dels (cdr dels))
@@ -289,7 +287,7 @@ destination is the same as that of the current message are processed."
       (message "This command cannot be used in this folder"))
      ((mew-folder-remotep folder)
       (message "Refiling and deleting...")
-      (sit-for 0)
+      (force-mode-line-update)
       (mew-summary-exec-remote beg end))
      ((mew-virtual-p)
       (if (not (mew-thread-p))
@@ -309,7 +307,7 @@ destination is the same as that of the current message are processed."
 		      "\\<mew-summary-mode-map>\\[mew-summary-exec]"))))))
      (t
       (message "Refiling and deleting...")
-      (sit-for 0)
+      (force-mode-line-update)
       (mew-summary-exec-local beg end func)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -421,10 +419,13 @@ destination is the same as that of the current message are processed."
 (defun mew-summary-exec-remote-get-info (beg end modify)
   (let* ((case (mew-sinfo-get-case))
 	 (folder (mew-sinfo-get-folder))
-	 (trash (mew-imap-trash-folder case))
+	 (trash (if (mew-folder-imapp folder) (mew-imap-trash-folder case)))
+	 (lomov (if (mew-folder-imapp folder) mew-imap-cache-refile-online))
+	 (reset (and modify (not lomov)))
 	 (m (make-marker))
-	 siz msg uid ref refs rmvs flds del regex kils
-	 refinfo refile-back)
+	 siz msg uid ref flds del regex trss
+	 rmvs kils refs movs
+	 refinfo refile-back mark rttl dttl)
     (if (and trash (string= trash folder)) (setq trash nil))
     (save-excursion
       (set-marker m end)
@@ -432,18 +433,23 @@ destination is the same as that of the current message are processed."
       (setq regex (mew-mark-regex mew-mark-refile))
       (goto-char beg)
       (while (re-search-forward regex m t)
-	(setq msg (mew-summary-message-number))
-	(setq uid (mew-summary-message-uid))
-	(setq siz (mew-summary-message-size))
+	(re-search-forward mew-regex-sumsyn-long)
+	(setq msg (mew-sumsyn-message-number))
+	(setq uid (mew-sumsyn-message-uid))
+	(setq siz (mew-sumsyn-message-size))
 	(setq refinfo (mew-refile-get msg))
 	(setq refile-back (cons refinfo refile-back))
 	(setq flds (cdr refinfo))
-	(if modify (mew-refile-reset msg)) ;; reset before delete just in case
+	;; 
+	(if reset (mew-refile-reset msg)) ;; reset before delete just in case
 	(setq del nil)
 	(if (member folder flds)
 	    (setq flds (delete folder flds)) ;; destructive
 	  (setq del t)
-	  (if modify (setq kils (cons msg kils))))
+	  (when modify
+	    (if lomov
+		(setq movs (cons msg movs))
+	      (setq kils (cons msg kils)))))
 	(when flds
 	  (setq ref (cons uid (cons siz (cons del flds))))
 	  (setq refs (cons ref refs)))
@@ -453,30 +459,41 @@ destination is the same as that of the current message are processed."
 	      (mew-summary-refile-remove-body)
 	      (mew-mark-remove))))
       (setq refs (nreverse refs))
+      (setq rttl (length refs))
+      (setq movs (nreverse movs))
       (if modify (mew-sinfo-set-refile-back refile-back))
+      (when lomov
+	(let ((mew-inherit-exec-case case))
+	  (mew-mark-sanity-refile movs)))
       ;;
       (setq regex (mew-mark-list-regex (list mew-mark-delete mew-mark-unlink)))
       (goto-char beg)
+      (setq dttl 0)
       (while (re-search-forward regex m t)
-	(setq msg (mew-summary-message-number))
-	(setq uid (mew-summary-message-uid))
-	(setq siz (mew-summary-message-size))
-	(if trash
-	    (setq rmvs (cons (list uid siz t trash) rmvs))
+	(setq mark (match-string 0))
+	(re-search-forward mew-regex-sumsyn-long)
+	(setq msg (mew-sumsyn-message-number))
+	(setq uid (mew-sumsyn-message-uid))
+	(setq siz (mew-sumsyn-message-size))
+	(if (and trash (char-equal (string-to-char mark) mew-mark-delete))
+	    (setq trss (cons (list uid siz t trash) trss))
 	  (setq rmvs (cons uid rmvs)))
 	(when modify
 	  (setq kils (cons msg kils))
-	  (mew-mark-invisible-line)))
+	  (mew-mark-invisible-line))
+	(setq dttl (1+ dttl)))
       (setq rmvs (nreverse rmvs))
-      (when trash
-	(setq refs (nconc rmvs refs))
-	(setq rmvs nil))
+      (setq trss (nreverse trss))
+      (when trss
+	(setq refs (nconc trss refs)))
       (set-buffer-modified-p nil)
-      (if modify
-	  (progn
-	    (sit-for 0)
-	    (list refs rmvs kils))
-	(list refs rmvs)))))
+      (cond
+       ((and (null rmvs) (null refs)) nil)
+       (modify
+	(force-mode-line-update) ;; xxx
+	(list rmvs kils refs movs rttl dttl))
+       (t
+	(list rmvs nil refs nil 0 0))))))
 
 (defun mew-summary-exec-remote (beg end)
   (if (mew-folder-nntpp (mew-sinfo-get-folder))
@@ -486,19 +503,16 @@ destination is the same as that of the current message are processed."
 	     (folder (mew-sinfo-get-folder))
 	     (bnm (mew-summary-folder-name 'ext))
 	     (lbuf (mew-refile-log-buffer bnm))
-	     (info (mew-summary-exec-remote-get-info beg end 'modify))
-	     (refs (nth 0 info))
-	     (rmvs (nth 1 info))
-	     (kils (nth 2 info)))
-	(if (or refs rmvs)
-	    (progn
-	      (cond
-	       ((mew-folder-popp folder)
-		(mew-pop-retrieve case 'exec bnm refs rmvs kils))
-	       ((mew-folder-imapp folder)
-		(mew-imap-retrieve case 'exec bnm refs rmvs kils)))
-	      (mew-exec-refile-log lbuf))
-	  (message "No messages to be processed"))))))
+	     (info (mew-summary-exec-remote-get-info beg end 'modify)))
+	(if (not info)
+	    (message "No messages to be processed")
+	  (mew-summary-reset)
+	  (cond
+	   ((mew-folder-popp folder)
+	    (mew-pop-retrieve case 'exec bnm info))
+	   ((mew-folder-imapp folder)
+	    (mew-imap-retrieve case 'exec bnm info)))
+	  (mew-exec-refile-log lbuf))))))
 
 (defun mew-mark-kill-invisible ()
   (save-excursion
@@ -535,66 +549,62 @@ the queue, type '\\[mew-summary-send-message]' in the queue online."
 	(setq beg (point-min))
 	(setq end (point-max))))
       (message "Refiling and deleting...")
-      (sit-for 0)
+      (force-mode-line-update)
       (mew-summary-exec-offline-region beg end))))
 
 (defun mew-summary-exec-offline-region (beg end)
   "Process the current marked message offline."
   (interactive)
   (let* ((info (mew-summary-exec-remote-get-info beg end nil))
-	 (refs (nth 0 info))
-	 (rmvs (nth 1 info)))
-    (if (or refs rmvs)
-	(let* ((fld (mew-sinfo-get-folder))
-	       (case (mew-sinfo-get-case))
-	       (imapq (mew-folder-case-folder
-		       case (mew-imap-queue-folder case)))
-	       (imapq-dir (mew-expand-folder imapq))
-	       file file-info buf ref)
-	  (unless (file-directory-p imapq-dir) (mew-make-directory imapq-dir))
-	  (setq file (mew-folder-new-message imapq))
-	  (setq file-info (concat file mew-imapq-info-suffix))
-	  (mew-lisp-save file-info (cons fld info) 'nobackup 'unlimit)
-	  (setq buf (find-file-noselect file))
-	  (save-excursion
-	    (set-buffer buf)
-	    (mew-header-insert mew-subj: (format "IMAP jobs for %s" fld))
-	    (mew-header-insert mew-date: (mew-time-ctz-to-rfc (current-time)))
-	    (mew-header-insert mew-from: "Mew IMAP manager")
-	    (insert "\n")
-	    (insert "Messages to be refiled:\n")
-	    (while refs
-	      (setq ref (car refs))
-	      (setq refs (cdr refs))
-	      (if (nth 2 ref)
-		  (insert (format "\t%s will be copied to %s\n"
-				  (nth 0 ref)
-				  (mew-join "," (nthcdr 3 ref))))
-		(insert (format "\t%s will be moved to %s\n"
+	 (rmvs (nth 0 info))
+	 (refs (nth 2 info)))
+    (if (not info)
+	(message "No messages to be processed")
+      (let* ((fld (mew-sinfo-get-folder))
+	     (case (mew-sinfo-get-case))
+	     (imapq (mew-folder-case-folder
+		     case (mew-imap-queue-folder case)))
+	     (imapq-dir (mew-expand-folder imapq))
+	     file file-info ref)
+	(unless (file-directory-p imapq-dir) (mew-make-directory imapq-dir))
+	(setq file (mew-folder-new-message imapq))
+	(setq file-info (concat file mew-imapq-info-suffix))
+	(mew-lisp-save file-info (cons fld info) 'nobackup 'unlimit)
+	(with-temp-buffer
+	  (mew-header-insert mew-subj: (format "IMAP jobs for %s" fld))
+	  (mew-header-insert mew-date: (mew-time-ctz-to-rfc (current-time)))
+	  (mew-header-insert mew-from: "Mew IMAP manager")
+	  (insert "\n")
+	  (insert "Messages to be refiled:\n")
+	  (while refs
+	    (setq ref (car refs))
+	    (setq refs (cdr refs))
+	    (if (nth 2 ref)
+		(insert (format "\t%s will be copied to %s\n"
 				(nth 0 ref)
-				(mew-join "," (nthcdr 3 ref))))))
-	    (insert "\n")
-	    (when rmvs
-	      (insert "Messages to be deleted:\n")
-	      (insert "\t" (mew-join "," rmvs) "\n"))
-	    (set-buffer-modified-p nil)
-	    (mew-frwlet
-	     mew-cs-dummy nil
-	     (write-region (point-min) (point-max) (buffer-file-name) nil 'no-msg)))
-	  (mew-touch-folder imapq)
-	  (mew-remove-buffer buf)
-	  ;; refile log is not necessary since mew-summary-exec-local does it.
-	  (if (mew-imap-trash-folder case)
-	      (let* ((mew-inherit-exec-case case)
-		     (mew-msg-rm-policy 'trashonly)
-		     (mew-trash-folder (mew-imap-trash-folder case))
-		     (mew-inherit-offline t))
-		(mew-summary-exec-local beg end))
-	    (let ((mew-inherit-exec-case case)
-		  (mew-msg-rm-policy 'always)
-		  (mew-inherit-offline t))
-	      (mew-summary-exec-local beg end))))
-      (message "No messages to be processed"))))
+				(mew-join "," (nthcdr 3 ref))))
+	      (insert (format "\t%s will be moved to %s\n"
+			      (nth 0 ref)
+			      (mew-join "," (nthcdr 3 ref))))))
+	  (insert "\n")
+	  (when rmvs
+	    (insert "Messages to be deleted:\n")
+	    (insert "\t" (mew-join "," rmvs) "\n"))
+	  (mew-frwlet
+	   mew-cs-dummy nil
+	   (write-region (point-min) (point-max) file nil 'no-msg)))
+	(mew-touch-folder imapq)
+	;; refile log is not necessary since mew-summary-exec-local does it.
+	(if (mew-imap-trash-folder case)
+	    (let* ((mew-inherit-exec-case case)
+		   (mew-msg-rm-policy 'trashonly)
+		   (mew-trash-folder (mew-imap-trash-folder case))
+		   (mew-inherit-offline t))
+	      (mew-summary-exec-local beg end))
+	  (let ((mew-inherit-exec-case case)
+		(mew-msg-rm-policy 'always)
+		(mew-inherit-offline t))
+	    (mew-summary-exec-local beg end)))))))
 
 (defun mew-summary-exec-offline-one (&optional arg)
   (interactive "P")
@@ -611,7 +621,7 @@ the queue, type '\\[mew-summary-send-message]' in the queue online."
           (setq end (1+ (point))))
         (if (> end (point-max))
             (message "No message")
-          (sit-for 0)
+	  (force-mode-line-update) ;; xxx
           (mew-summary-exec-offline-region beg end))))))
 
 (provide 'mew-exec)
