@@ -57,8 +57,7 @@
     ("select"        ("OK" . "uid"))
     ("select2"       ("OK" . "search"))
     ("uid"           ("OK" . "umsg"))
-    ;; xxx what if "NO", not "NO [TRYCREATE]"?
-    ("copy"          ("OK" . "copy") ("\\[TRYCREATE\\]" . "create"))
+    ("copy"          ("OK" . "copy") ("NO \\[TRYCREATE\\]" . "create") ("NO" . "wmbx"))
     ("create"        ("OK" . "copy") ("NO" . "wmbx"))
     ("dels"	     ("OK" . "dels"))
     ("expunge"       ("OK" . "post-expunge"))
@@ -138,18 +137,10 @@
     (mew-imap-process-send-string pro pnm "LOGIN \"%s\" \"%s\"" user pass)))
 
 (defun mew-imap-command-wpwd (pro pnm)
-  (let* ((directive (mew-imap-get-directive pnm))
-	 (bnm (mew-imap-get-bnm pnm)))
-    (mew-imap-message pnm "IMAP password is wrong!")
-    (mew-passwd-set-passwd (mew-imap-passtag pnm) nil)
-    (when (eq directive 'exec)
-      (mew-summary-visible-buffer bnm)
-      (save-excursion
-	(set-buffer bnm)
-	(mew-sinfo-set-refile
-	 (nconc (mew-sinfo-get-refile) (mew-sinfo-get-refile-back)))
-	(mew-sinfo-set-refile-back nil)))
-    (mew-imap-command-logout2 pro pnm)))
+  (mew-imap-set-error pnm "IMAP password is wrong!")
+  (mew-passwd-set-passwd (mew-imap-passtag pnm) nil)
+  (mew-imap-set-status pnm "logout2")
+  (mew-imap-command-logout2 pro pnm))
 
 (defun mew-imap-command-next (pro pnm)
   (let ((directive (mew-imap-get-directive pnm))
@@ -420,9 +411,9 @@
     (mew-imap-process-send-string pro pnm "CREATE \"%s\"" dst)))
 
 (defun mew-imap-command-wmbx (pro pnm)
-  (mew-imap-set-error pnm "mailbox is wrong!")
-  (mew-imap-set-status pnm "logout")
-  (mew-imap-command-logout pro pnm))
+  (mew-imap-set-error pnm "Mailbox name is wrong!")
+  (mew-imap-set-status pnm "logout2")
+  (mew-imap-command-logout2 pro pnm))
 
 (defun mew-imap-command-pre-dels (pro pnm)
   (let ((dtt (length (mew-imap-get-dels pnm)))
@@ -827,8 +818,6 @@
 (defun mew-imap-command-logout2 (pro pnm)
   (let ((wrk (mew-imap-get-wrk pnm)))
     (if wrk (mew-queue-enqueue2 wrk)))
-  (mew-imap-set-done pnm t)
-  (mew-imap-set-error pnm t)
   (mew-imap-process-send-string pro pnm "LOGOUT"))
 
 (defun mew-imap-command-logout3 (pro pnm)
@@ -1254,8 +1243,8 @@
   (let* ((pnm (process-name process))
 	 (status (mew-imap-get-status pnm))
 	 (tag (mew-imap-get-tag pnm))
-	 (eos (format "^%s \\(OK\\|NO\\|BAD\\)\\(.*\\)" tag))
-	 stay next func code resp)
+	 (eos (format "^%s \\(OK\\|NO \\[TRYCREATE\\]\\|NO\\|BAD\\)" tag))
+	 stay next func code)
     (mew-imap-debug (upcase status) string)
     (mew-filter
      ;; Process's buffer
@@ -1280,9 +1269,7 @@
       ((and (goto-char (point-max)) (= (forward-line -1) 0) (looking-at eos))
        (mew-imap-set-tag pnm nil)
        (setq code (mew-match-string 1))
-       (setq resp (mew-match-string 2))
-       (setq next (mew-imap-fsm-next status code))
-       (unless next (setq next (mew-imap-fsm-next status resp))))
+       (setq next (mew-imap-fsm-next status code)))
       (t
        (setq stay t)))
      (unless stay
@@ -1326,8 +1313,10 @@
 	(let* ((rtrs (mew-imap-get-rtrs pnm))
 	       (lefts (length rtrs))
 	       (msgid (nth 0 (car rtrs)))
-	       recovered)
-	  (mew-imap-message pnm "IMAP connection is lost")
+	       leftp)
+	  (if error
+	      (mew-imap-message pnm error)
+	    (mew-imap-message pnm "IMAP connection is lost"))
 	  (cond
 	   ((eq directive 'exec)
 	    (mew-imap-exec-recover bnm))
@@ -1338,23 +1327,20 @@
 	     ((eq directive 'scan)
 	      (setq msgid (int-to-string (1- (string-to-int msgid))))
 	      (mew-lisp-save file msgid nil 'unlimit)
-	      (setq recovered t))
+	      (setq leftp t))
 	     ((eq directive 'inc)
 	      ;; uidl is reversed.
 	      (setq msgid (assoc msgid uidl))
 	      (setq uidl (cdr (member msgid uidl)))
 	      (mew-net-uidl-db-set (mew-imap-passtag pnm) uidl)
-	      (setq recovered t)))
-	    (when recovered
+	      (setq leftp t)))
+	    (when leftp
 	      (mew-imap-message
 	       pnm
 	       "%d message retrieved. %d messages are left due to an error."
 	       (- rttl lefts) lefts)
 	      (mew-summary-folder-cache-save))))
       (cond
-       (error 
-	;; retain the error message
-	)
        ((eq directive 'list)
 	(mew-imap-message pnm "Collecting mailbox list...done"))
        ((eq directive 'delete)
