@@ -105,7 +105,7 @@
   (define-key mew-summary-mode-map "\C-c\C-r" 'mew-summary-isearch-backward)
   (define-key mew-summary-mode-map "\C-c\C-o" 
     'mew-summary-jump-to-draft-buffer)
-  (define-key mew-summary-mode-map "\ea"  'mew-summary-addrbook-edit)
+  (define-key mew-summary-mode-map "\ea"  'mew-summary-alias-edit)
   (define-key mew-summary-mode-map "\el"  'mew-summary-recenter)
   (define-key mew-summary-mode-map "\et"  'mew-summary-uudecode)
   (define-key mew-summary-mode-map "\es"  'mew-summary-unshar)
@@ -148,14 +148,18 @@
   (define-key mew-summary-mode-map "t\ed" 'mew-thread-mark-unlink)
   (define-key mew-summary-mode-map "tu"   'mew-thread-unmark)
   (define-key mew-summary-mode-map "tg"   'mew-thread-glue)
+  (define-key mew-summary-mode-map "tr"   'mew-summary-regexp-make-thread)
   (define-key mew-summary-mode-map "z"    (make-sparse-keymap))
   (define-key mew-summary-mode-map "z8"   'mew-summary-toggle-8bit)
   (define-key mew-summary-mode-map "zc"   'mew-summary-cache-clean-up)
   (define-key mew-summary-mode-map "zd"   'mew-summary-toggle-debug)
   (define-key mew-summary-mode-map "zp"   'mew-summary-toggle-policy)
+  (define-key mew-summary-mode-map "zv"   'mew-summary-toggle-header-veil)
   (define-key mew-summary-mode-map "zw"   'mew-summary-toggle-warning)
   (define-key mew-summary-mode-map "l"	  (make-sparse-keymap))
   (define-key mew-summary-mode-map "lc"   'mew-summary-local-copy)
+  (define-key mew-summary-mode-map "lh"   'mew-summary-learn-ham)
+  (define-key mew-summary-mode-map "ls"   'mew-summary-learn-spam)
   (define-key mew-summary-mode-map "="    'mew-summary-info)
   ;;
   ;; not provided in Virtual mode
@@ -177,12 +181,14 @@
   (define-key mew-summary-mode-map "mxd"   'mew-summary-exec-delete)
   (define-key mew-summary-mode-map "mx\ed" 'mew-summary-exec-unlink)
   (define-key mew-summary-mode-map "mxo"   'mew-summary-exec-refile)
-
   (define-key mew-summary-mode-map "s"    'mew-summary-ls)
   (define-key mew-summary-mode-map "O"    'mew-summary-pack)
   (define-key mew-summary-mode-map "S"    'mew-summary-sort)
   (define-key mew-summary-mode-map "\C-c\C-b" 'mew-summary-exchange-point)
   (define-key mew-summary-mode-map "\eo"  'mew-summary-auto-refile)
+  (define-key mew-summary-mode-map "R"	  (make-sparse-keymap))
+  (define-key mew-summary-mode-map "Rd"   'mew-summary-delete-folder)
+  (define-key mew-summary-mode-map "Rr"   'mew-summary-rename-folder)
   ;;
   (if mew-xemacs-p
       (define-key mew-summary-mode-map 'button2 'mew-summary-mouse-show)
