@@ -316,6 +316,8 @@ Binary search is used for speed reasons."
      (goto-char (point-max))
      (forward-line -1)
      (when (looking-at "^\\.$")
+       (unless mew-xemacs-p
+	 (process-send-string process "\n"))
        (mew-dot-delete)
        (goto-char (point-min))
        (unless first
@@ -333,8 +335,9 @@ Binary search is used for speed reasons."
        (mew-scan-body vec draftp)
        (mew-scan-insert-line bnm vec width nil mark)
        (delete-region (point-min) (point-max))
-       (mew-timing) ;; for XEmacs
-       (process-send-string process "\n")))))
+       (when mew-xemacs-p
+	 (mew-timing) ;; for XEmacs
+	 (process-send-string process "\n"))))))
 
 (defun mew-local-sentinel (process event)
   (let* ((pnm (process-name process))
