@@ -79,7 +79,7 @@
      ((and (listp program) (nth 0 program) (symbolp (nth 0 program)))
       (when (fboundp (nth 0 program))
 	(if (mew-ct-imagep ct)
-	    (funcall (nth 0 program) cache begin end params ct)
+	    (funcall (nth 0 program) cache begin end params fname ct cte)
 	  (if (eq (nth 0 program) 'mew-mime-application/octet-stream)
 	      (funcall (nth 0 program) cache begin end params ct cte fname (nth 1 program))
 	    (funcall (nth 0 program) cache begin end params)))))
@@ -474,7 +474,7 @@
 ;;; Image/*
 ;;;
 
-(defun mew-mime-image/* (cache begin end &optional params ct)
+(defun mew-mime-image/* (cache begin end &optional params fname ct cte)
   (let* ((format (mew-mime-image-format-name ct)))
     (if (mew-image-inline-p format)
 	(mew-mime-image cache begin end format)
@@ -488,6 +488,9 @@
 	       "  #####  #     # #     #  #####  #######\n"
 	       "\n\n")
        (mew-insert "Content-Type:\t%s\n" ct)
+       (mew-insert "Encoding:\t%s\n" cte)
+       (mew-insert "Size:\t\t%d bytes\n" (mew-region-bytes begin end cache))
+       (mew-insert "Filename:\t%s\n" fname)
        (mew-insert "Program:\t%s\n" mew-prog-image/*-ext)
        (insert "\n")
        (mew-mime-part-messages t)))))
@@ -644,7 +647,7 @@
 			   file1 
 			   (file-name-nondirectory file2)))
 	   (let ((buffer-file-coding-system)) ;; to prevent the side effect
-	     (insert-file-contents file2)))
+	     (mew-insert-file-contents file2)))
 	(mew-frwlet
 	 'utf-8 mew-cs-dummy
 	 (call-process prog nil (current-buffer) nil file1)))
