@@ -69,6 +69,10 @@ at completions."
 (defvar mew-input-folder-search-buf " *Mew* folder search")
 
 (defvar mew-input-folder-hist nil)
+
+(defvar mew-input-folder-circular-case nil)
+(defvar mew-input-folder-refile nil)
+
 (defvar mew-input-folder-search-direction nil)
 (defvar mew-input-folder-search-key nil)
 (defvar mew-input-folder-search-match nil)
@@ -122,7 +126,21 @@ at completions."
    (put-text-property (mew-minibuf-point-min) (point-max) 'read-only t)))
 
 (defun mew-input-folder-search-setup (&optional min)
-  (let* (case:fld case fld alist buf)
+  (if mew-input-folder-refile
+      (mew-input-folder-search-setup2 min)
+    (mew-input-folder-search-setup1 min)))
+
+(defun mew-input-folder-search-setup-buffer (alist min)
+  (save-excursion
+    (set-buffer (get-buffer-create mew-input-folder-search-buf))
+    (setq buffer-read-only t)
+    (mew-elet
+     (mew-erase-buffer)
+     (mapcar (lambda (x) (if (stringp (car x)) (insert (car x) "\n"))) alist))
+    (if min (goto-char (point-min)))))
+
+(defun mew-input-folder-search-setup1 (&optional min)
+  (let (case:fld case fld alist)
     (save-excursion
       (goto-char (point-max))
       (if (search-backward "," nil t)
@@ -134,11 +152,13 @@ at completions."
     (if mew-input-folder-search-point
 	(setq case:fld (substring case:fld mew-input-folder-search-point)))
     (setq case (car (mew-folder-case case:fld)))
+    (if case
+	(setq mew-input-folder-search-point
+	      (+ (or mew-input-folder-search-point 0) (length case) 1))) ;; ":"
     (setq fld (mew-folder-folder case:fld))
     (mew-input-folder-display)
     (if (or (null fld) (string= fld ""))
 	(setq fld (mew-proto case)))
-    (setq buf (get-buffer-create mew-input-folder-search-buf))
     (cond
      ((mew-folder-popp fld)
       (setq alist (mew-pop-folder-alist)))
@@ -148,13 +168,35 @@ at completions."
       (setq alist (mew-imap-folder-alist case)))
      (t
       (setq alist (mew-local-folder-alist))))
+    (mew-input-folder-search-setup-buffer alist min)))
+
+(defun mew-input-folder-search-setup2 (&optional min)
+  (let (do-search alist)
     (save-excursion
-      (set-buffer buf)
-      (setq buffer-read-only t)
-      (mew-elet
-       (mew-erase-buffer)
-       (mapcar (lambda (x) (if (stringp (car x)) (insert (car x) "\n"))) alist))
-      (if min (goto-char (point-min))))))
+      (goto-char (point-max))
+      (if (search-backward "," nil t)
+	  (progn
+	    (goto-char (match-end 0))
+	    (unless (looking-at mew-regex-case2)
+	      (setq do-search t)
+	      (setq mew-input-folder-search-point
+		    (- (point) (mew-minibuf-point-min)))))
+	(goto-char (mew-minibuf-point-min))
+	(unless (looking-at mew-regex-case2)
+	  (setq do-search t))))
+    (if (not do-search)
+	(progn
+	  (mew-temp-minibuffer-message "Remove case!")
+	  (mew-input-folder-clean-up))
+      (setq mew-input-folder-search-original
+	    (mew-buffer-substring (mew-minibuf-point-min) (point-max)))
+      (mew-input-folder-display)
+      (cond
+       ((eq mew-input-complete-function 'mew-complete-local-folder)
+	(setq alist (mew-local-folder-alist)))
+       ((eq mew-input-complete-function 'mew-complete-imap-folder)
+	(setq alist (mew-imap-folder-alist mew-inherit-case))))
+      (mew-input-folder-search-setup-buffer alist min))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -274,12 +316,11 @@ Otherwise, it exits minibuffer."
     (abort-recursive-edit)))
 
 (defun mew-input-folder-comma ()
-  "This function inserts ',' in a single folder input
-and does not insert anything in a multiple folder input."
+  "This function inserts ',' on multiple folder input
+and does not insert anything on single folder input."
   (interactive)
-  (if (and (null mew-input-folder-search-direction)
-	   mew-input-folder-search-multi)
-      (insert ",")))
+  (unless mew-input-folder-search-direction
+    (insert ",")))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -357,94 +398,220 @@ folder and displays it in addition to its bound key."
 ;;; Prefix hack
 ;;;
 
+(defun mew-input-folder-complete-case-input (prefix-char)
+  (when (and mew-use-case-input-completion
+	     (eq mew-input-complete-function 'mew-complete-folder)
+	     (mew-folder-remotep (char-to-string prefix-char))
+	     (and mew-case-input
+		  (not (string= mew-case-input mew-case-default))))
+    (insert mew-case-input ":")))
+
 (defun mew-input-folder-prefix ()
   "A function to insert a folder prefix in minibuffer.
 If the previous character is another folder prefix, 
 it is deleted automatically."
   (interactive)
-  (if mew-input-folder-search-direction
-      (mew-input-folder-self-insert)
+  (cond
+   (mew-input-folder-search-direction
+    (mew-input-folder-self-insert))
+   ;;
+   ((or (eq mew-input-complete-function 'mew-complete-local-folder)
+	(eq mew-input-complete-function 'mew-complete-imap-folder))
+    ;; no case, not allow other prefixes.
+    (let* ((pos (- (point) (mew-minibuf-point-min)))
+	   (prefix (if (eq mew-input-complete-function
+			   'mew-complete-local-folder)
+		       mew-folder-local mew-folder-imap))
+	   (insert-ok (string= (char-to-string last-command-char) prefix)))
+      (cond
+       ((or (= pos 0)
+	    (save-excursion (forward-char -1) (looking-at ",")))
+	(if insert-ok (insert prefix)))
+       ((or (and (= pos 1)
+		 (save-excursion (forward-char -1) (looking-at "[-+%$]"))) 
+	    ;; excluding *
+	    (save-excursion (forward-char -2) (looking-at ",[-+%$]")))
+	;; A wrong prefix might accidentally be here. 
+	;; So, replace it just in case
+	(when insert-ok
+	  (forward-char -1)
+	  (delete-char 1)
+	  (insert prefix)))
+       (t
+	(insert last-command-char)))))
+   (t
     (let ((pos (- (point) (mew-minibuf-point-min))))
       (cond
-       ((= pos 0)
+       ((or (= pos 0)
+	    (save-excursion (forward-char -1) (looking-at ",")))
+	(mew-input-folder-complete-case-input last-command-char)
 	(insert last-command-char))
-       ((and (= pos 1)
-	     (save-excursion
-	       (forward-char -1)
-	       (looking-at "[-+%$]"))) ;; excluding *
+       ;;
+       ((or (and (= pos 1)
+		 (save-excursion (forward-char -1) (looking-at "[-+%$]"))) 
+	    ;; excluding *
+	    (save-excursion (forward-char -2) (looking-at ",[-+%$]")))
 	(forward-char -1)
 	(delete-char 1)
+	(mew-input-folder-complete-case-input last-command-char)
 	(insert last-command-char))
-       ((and (> pos 1)
-	     (save-excursion
-	       (forward-char -2)
-	       (looking-at "[:,][-+*%$]")))
+       ;;
+       ((save-excursion
+	  (forward-char -2)
+	  (looking-at ":[-+*%$]"))
 	(forward-char -1)
 	(delete-char 1)
+	(when (memq last-command-char '(?+ ?*))
+	  ;; delete case:
+	  (delete-region (save-excursion (or (and (search-backward "," nil t)
+						  (match-end 0))
+					     (mew-minibuf-point-min)))
+			 (point)))
 	(insert last-command-char))
+       ;;
        (t
-	(insert last-command-char))))))
+	(insert last-command-char)))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Folder
 ;;;
 
-(defun mew-input-folder (default &optional local-only)
-  "Input a folder from the minibuffer.
-DEFAULT must be a folder WITHOUT case.
-This function returns a folder."
-  (mew-input-clear)
-  (mew-input-folder-clean-up)
-  (setq mew-input-folder-search-multi nil)
-  (let ((prefix (mew-folder-prefix default))
-	(mew-inherit-case (mew-sinfo-get-case))
-	(mew-input-complete-function
-	 (if local-only 'mew-complete-local-folder 'mew-complete-folder))
-	folder)
-    (setq folder (read-from-minibuffer (format "Folder name (%s): " default)
-				       prefix mew-input-folder-map nil
-				       'mew-input-folder-hist))
-    (when (or (string= folder "") (member folder mew-folder-prefixes))
-      (setq folder default))
-    ;; +foo/ -> +foo
-    (directory-file-name folder)))
+(defun mew-input-folder-check (folders)
+  (let (case:folder folder ret)
+    (while folders
+      (setq case:folder (car folders))
+      (setq folders (cdr folders))
+      (catch 'continue
+	(if (string-match mew-regex-case case:folder)
+	    (progn
+	      (setq folder (mew-match-string 2 case:folder))
+	      (if (= (length folder) 0) (throw 'continue nil))) ;; "case:" only
+	  (setq folder case:folder))
+	(if (and (= (length folder) 1)
+		 (member folder mew-folder-prefixes)) ;; "prefix" only
+	    (throw 'continue nil))
+	(if (and (mew-folder-popp folder) ;; not $inbox
+		 (not (string= mew-pop-inbox-folder folder)))
+	    (throw 'continue nil))
+	(when (string-match " " folder)
+	  (mew-warn "SPC is not allowed!")
+	  (throw 'continue nil))
+	(setq case:folder (mew-canonicalize-case-folder case:folder))
+	(setq ret (cons case:folder ret))))
+    (nreverse ret)))
 
-(defun mew-input-folders (default &optional proto)
-  ;; Returns a list
+(defun mew-input-local-folder (folder)
+  "Input a local folder from the minibuffer."
   (mew-input-clear)
   (mew-input-folder-clean-up)
-  (setq mew-input-folder-search-multi t)
-  (let ((mew-input-complete-function (if proto
-					 (if (mew-folder-imapp proto)
-					     'mew-complete-imap-folder
-					   'mew-complete-local-folder)
-				       'mew-complete-folder))
-	(mew-inherit-case (mew-sinfo-get-case))
-	prompt init folders protos)
+  (let* ((default folder)
+	 (init (mew-folder-prefix folder))
+	 (mew-input-complete-function 'mew-complete-local-folder)
+	 (mew-input-folder-circular-case t)
+	 (ret (read-from-minibuffer (format "Folder name (%s): " default)
+				    init mew-input-folder-map nil
+				    'mew-input-folder-hist)))
+    (when (or (string= ret "") (string= ret init))
+      (setq ret default))
+    (car (mew-input-folder-check (list ret)))))
+
+(defun mew-input-folder (case folder)
+  "Input a folder from the minibuffer."
+  (mew-input-clear)
+  (mew-input-folder-clean-up)
+  (let* ((default (mew-folder-case-folder case folder))
+	 (init (mew-folder-case-folder case (mew-folder-prefix folder)))
+	 (mew-input-complete-function 'mew-complete-folder)
+	 (mew-input-folder-circular-case t)
+	 ;; mew-inherit-case must be nil
+	 (ret (read-from-minibuffer (format "Folder name (%s): " default)
+				    init mew-input-folder-map nil
+				    'mew-input-folder-hist)))
+    (when (or (string= ret "") (string= ret init))
+      (setq ret default))
+    (car (mew-input-folder-check (list ret)))))
+
+(defun mew-input-folders (case:folder)
+  "Input a folder from the minibuffer."
+  (mew-input-clear)
+  (mew-input-folder-clean-up)
+  (let* ((init (mew-canonicalize-case-folder case:folder))
+	 (mew-input-complete-function 'mew-complete-folder)
+	 (mew-input-folder-circular-case t)
+	 (mew-input-folder-search-multi t)
+	 ;; mew-inherit-case must be nil
+	 (ret (read-from-minibuffer "Folder name: "
+				    init mew-input-folder-map nil
+				    'mew-input-folder-hist)))
+    (when (string= ret "")
+      (setq ret init))
+    (setq ret (mapcar 'mew-chop (mew-split ret ?,)))
+    (mew-input-folder-check ret)))
+
+(defun mew-input-refile-folder-check (folders proto)
+  (let (case:folder folder ret)
+    (while folders
+      (setq case:folder (car folders))
+      (setq folders (cdr folders))
+      (catch 'continue
+	;; If case is specified, just remove it at this moment...
+	(if (string-match mew-regex-case case:folder)
+	    (progn
+	      (setq folder (mew-match-string 2 case:folder))
+	      (if (= (length folder) 0) (throw 'continue nil))) ;; "case:" only
+	  (setq folder case:folder))
+	(cond
+	 ((eq proto 'imap)
+	  (unless (mew-folder-imapp folder)
+	    (throw 'continue nil)))
+	 ((eq proto 'local)
+	  (unless (mew-folder-localp folder)
+	    (throw 'continue nil))))
+	(if (and (= (length folder) 1)
+		 (member folder mew-folder-prefixes)) ;; "prefix" only
+	    (throw 'continue nil))
+	(when (string-match " " folder)
+	  (mew-warn "SPC is not allowed!")
+	  (throw 'continue nil))
+	(setq folder (mew-canonicalize-case-folder folder))
+	(setq ret (cons folder ret))))
+    (nreverse ret)))
+
+(defun mew-input-refile-folders (folder-list singlep case proto)
+  "Input refile folders from the minibuffer."
+  (mew-input-clear)
+  (mew-input-folder-clean-up)
+  (let ((mew-input-complete-function (if (mew-folder-imapp proto)
+					 'mew-complete-imap-folder
+				       'mew-complete-local-folder))
+	(mew-inherit-case case)
+	(mew-input-folder-search-multi t)
+	(mew-input-folder-refile t)
+	;; Emacs 21.1, 21.2 and 21.3 has a bug of inhibit-quit.
+	;; Set inhibit-quit to nil so that C-g can be used
+	(inhibit-quit nil)
+	default prompt init ret)
     (cond
-     ;; Must be without case.
-     ((stringp default)
-      (setq prompt (format "Folder name (%s): " default))
-      (setq init (mew-folder-prefix default)))
+     (singlep
+      (setq default (car folder-list))
+      (setq init (mew-folder-prefix default))
+      (if case
+	  (setq prompt (format "Folder name <%s:> (%s): " case default))
+	(setq prompt (format "Folder name (%s): " default))))
      (t
-      (setq prompt "Folder name: ")
-      (setq init (mew-join "," default))))
-    (setq folders (read-from-minibuffer prompt
-					init mew-input-folder-map nil
-					'mew-input-folder-hist))
-    (when (and (stringp default) (string= folders init))
-      (setq folders default))
-    (setq folders (mapcar 'mew-chop (mew-split folders ?,)))
-    (when folders
-      (setq protos mew-folder-prefixes)
-      (while protos
-	(setq folders (delete (car protos) folders))
-	(setq protos (cdr protos))))
-    (if folders
-	;; +foo/ -> +foo
-	(mapcar 'directory-file-name folders)
-      nil)))
+      (if case
+	  (setq prompt (format "Folder name <%s:>: " case))
+	(setq prompt "Folder name: "))
+      (setq init (mew-join "," folder-list))))
+    (setq ret (read-from-minibuffer prompt
+				    init mew-input-folder-map nil
+				    'mew-input-folder-hist))
+    (when (and singlep (string= ret init))
+      (setq ret default))
+    (setq ret (mapcar 'mew-chop (mew-split ret ?,)))
+    (mew-input-refile-folder-check
+     ret (if (mew-folder-imapp proto) 'imap 'local))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -527,7 +694,7 @@ This function returns a folder."
   (let* ((mew-input-complete-function 'mew-complete-sort-key)
 	 (mew-input-exit-minibuffer-function 'mew-input-sort-key-check)
 	 (field:mode (read-from-minibuffer
-		      (format "Sort by (%s)? : " key)
+		      (format "Sort by (%s)?: " key)
 		      ""
 		      mew-input-map
 		      nil 
@@ -660,7 +827,7 @@ This function returns a folder."
 
 (defun mew-input-directory-name (&optional default)
   (let ((dir (expand-file-name
-	      (read-file-name "Directory : " default default t))))
+	      (read-file-name "Directory: " default default t))))
     (if (file-directory-p dir)
 	dir
       (mew-warn "%s is not directory" dir)
@@ -726,8 +893,8 @@ This function returns a folder."
 
 (defun mew-input-general (prompt alist &optional require-match initial)
   (let* ((completion-ignore-case t)
-	 (question (if initial (format "%s (%s) : " prompt initial)
-		     (format "(%s) : " prompt)))
+	 (question (if initial (format "%s (%s): " prompt initial)
+		     (format "(%s): " prompt)))
 	 (value (completing-read question alist nil require-match nil)))
     (if (and initial (string= value "")) initial value)))
 
@@ -767,6 +934,8 @@ This function returns a folder."
           nil)
       t)))
 
+(defvar mew-input-case-hist nil)
+
 (defun mew-input-case (default msg &optional edit)
   (mew-input-clear)
   (unless default (setq default mew-case-default))
@@ -778,11 +947,15 @@ This function returns a folder."
 	(setq case (read-from-minibuffer
 		    (format "%s case value: " msg)
 		    default
-		    mew-input-map))
+		    mew-input-map
+                    nil
+                    'mew-input-case-hist))
       (setq case (read-from-minibuffer
 		  (format "%s case value (%s): " msg default)
 		  ""
-		  mew-input-map)))
+		  mew-input-map
+                  nil
+                  'mew-input-case-hist)))
     (if (string= case "")
 	default
       (setq lst (mew-split case ?,))
@@ -801,11 +974,11 @@ This function returns a folder."
   (let ((ociea cursor-in-echo-area) char)
     (unwind-protect
 	(progn
-	  (message "Input mark : ")
+	  (message "Input mark: ")
 	  (setq cursor-in-echo-area t)
 	  (setq char (read-char))
 	  (unless (char-equal char ?\r)
-	    (message "Input mark : %s" (char-to-string char))))
+	    (message "Input mark: %s" (char-to-string char))))
       (setq cursor-in-echo-area ociea))
     (cond
      ((char-equal char ?\r) char)
@@ -875,11 +1048,15 @@ This function returns a folder."
     (mew-read-passwd prompt)))
 
 (defun mew-read-passwd (prompt)
-  (let ((inhibit-input-event-recording t))
+  (let ((inhibit-input-event-recording t)
+	;; A process filter sets inhibit-quit to t to prevent quitting.
+	;; Set inhibit-quit to nil so that C-g can be used
+	(inhibit-quit nil))
     (condition-case nil
 	(read-passwd prompt)
       ;; If read-passwd causes an error, let's return "" so that
       ;; the password process will safely fail.
+      (quit "")
       (error ""))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -904,7 +1081,7 @@ This function returns a folder."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2002 Mew developing team.
+;; Copyright (C) 1997-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
