@@ -85,11 +85,11 @@
 	(setq mew-summary-buffer-process-status (format " %s" status)))
       (setq mew-summary-buffer-secure-process secure))))
 
-(defun mew-net-status1 (buf dttl dcnt &optional secure)
+(defun mew-net-status1 (buf msg ttl cnt &optional secure)
   (when mew-use-net-status
-    (when (<= dcnt dttl)
-      (let ((substatus (format "%d/%d" dcnt dttl)))
-	(mew-net-status buf "Deleting" substatus secure)))))
+    (when (<= cnt ttl)
+      (let ((substatus (format "%02d%%" (/ (* cnt 100) ttl))))
+	(mew-net-status buf msg substatus secure)))))
 
 (defvar mew-net-status-percent-size 51200)
 
@@ -280,8 +280,9 @@
     (rename-file work orig 'override)
     (rename-file iwrk info 'override)))
 
-(defun mew-queue-insert-file (pnm n file) ;; 1
-  (let* ((work (concat file mew-queue-work-suffix)) ;; 1.wrk
+(defun mew-queue-insert-file (pnm n qfld msg) ;; 1
+  (let* ((file (mew-expand-folder qfld msg))
+	 (work (concat file mew-queue-work-suffix)) ;; 1.wrk
 	 (info (concat file mew-queue-info-suffix)) ;; 1.mqi
 	 (iwrk (concat info mew-queue-work-suffix)) ;; 1.mqi.wrk
 	 (i 0) buf data)
@@ -291,7 +292,7 @@
       (if (setq buf (get-file-buffer work)) (mew-remove-buffer buf))
       (mew-frwlet
        mew-cs-text-for-read mew-cs-dummy
-       (set-buffer (find-file-noselect work)))
+       (set-buffer (mew-find-file-noselect work)))
       (setq data (mew-lisp-load info))
       (while (< i n)
 	(aset (mew-info pnm) i (aref data i))
@@ -299,12 +300,12 @@
       (rename-file info iwrk 'override)
       t)))
 
-(defun mew-queue-get-next (pnm msgs n func)
+(defun mew-queue-get-next (pnm qfld msgs n func)
   (let* (msg flushp)
     (while (and msgs (not flushp))
       (setq msg (car msgs))
       (setq msgs (cdr msgs))
-      (if (mew-queue-insert-file pnm n msg) (setq flushp t)))
+      (if (mew-queue-insert-file pnm n qfld msg) (setq flushp t)))
     (if (fboundp func) (funcall func pnm msgs)) ;; set-messages
     flushp))
 
@@ -323,8 +324,15 @@
 ;;; Fcc:
 ;;;
 
-(defun mew-net-fcc-message (flds file)
-  (let (fcc-file link fld)
+(defun mew-net-fcc-message (case fcc file)
+  (let (flds fcc-file link fld imapp)
+    (while fcc
+      (if (mew-folder-imapp (car fcc))
+	  (progn
+	    (setq flds (cons (mew-folder-imap-to-fcc case (car fcc)) flds))
+	    (setq imapp t))
+	(setq flds (cons (car fcc) flds)))
+      (setq fcc (cdr fcc)))
     (catch 'loop
       (while flds
 	(setq fld (car flds))
@@ -343,7 +351,8 @@
       (setq link (mew-folder-new-message fld))
       (when link
 	(mew-link fcc-file link)
-	(mew-touch-folder fld)))))
+	(mew-touch-folder fld)))
+    imapp))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -419,30 +428,33 @@ If 'mew-auto-flush-queue' is non-nil, +queue is flushed.
 If called with '\\[universal-argument]', +queue is not flushed."
   (interactive "P")
   (let* ((case mew-case-input)
+	 (dont (not (mew-use-retrieve case)))
 	 (inbox (mew-inbox-folder case))
 	 (dir (mew-expand-folder inbox))
 	 (mailbox (mew-mailbox-type case)))
-    (unless (file-directory-p dir)
-      (mew-make-directory dir))
-    ;; for mew-summary-exchange-point.
-    (set-marker mew-summary-inbox-position (point) (current-buffer))
-    (mew-summary-switch-to-folder inbox)
-    ;; in the inbox
-    (when (mew-summary-exclusive-p)
-      (mew-summary-reset)
-      ;;
-      (mew-summary-retrieve-gap inbox)
-      ;;
-      (goto-char (point-max))
-      (mew-sinfo-set-start-point (point))
-      (mew-sinfo-set-direction 'down)
-      (cond
-       ((eq mailbox 'pop)
-	(mew-pop-retrieve case 'inc inbox (not no-flush)))
-       ((eq mailbox 'imap)
-	(mew-imap-retrieve case 'inc inbox (not no-flush)))
-       ((eq mailbox 'mbox)
-	(mew-mbox-retrieve case 'inc inbox (not no-flush)))))))
+    (if dont
+	(message "This command is not allowed to use")
+      (unless (file-directory-p dir)
+	(mew-make-directory dir))
+      ;; for mew-summary-exchange-point.
+      (set-marker mew-summary-inbox-position (point) (current-buffer))
+      (mew-summary-switch-to-folder inbox)
+      ;; in the inbox
+      (when (mew-summary-exclusive-p)
+	(mew-summary-reset)
+	;;
+	(mew-summary-retrieve-gap inbox)
+	;;
+	(goto-char (point-max))
+	(mew-sinfo-set-start-point (point))
+	(mew-sinfo-set-direction 'down)
+	(cond
+	 ((eq mailbox 'pop)
+	  (mew-pop-retrieve case 'inc inbox (not no-flush)))
+	 ((eq mailbox 'imap)
+	  (mew-imap-retrieve case 'inc inbox (not no-flush)))
+	 ((eq mailbox 'mbox)
+	  (mew-mbox-retrieve case 'inc inbox (not no-flush))))))))
 
 (defun mew-summary-scan-boot (proto case)
   ;; called by mew()
@@ -659,7 +671,9 @@ The messages in the server side is always retained."
     (nreverse ret)))
 
 (defun mew-net-msg-group (lst)
-  (mew-net-msg-cat (mew-net-msg-pack lst)))
+  (if (= (length lst) 1)
+      lst ;; "1:*"
+    (mew-net-msg-cat (mew-net-msg-pack lst))))
 
 (provide 'mew-net)
 
