@@ -18,14 +18,15 @@
     "case" ;; save for re-edit, not for sending
     ;; parameters used internally
     "server" "port" "ssh-server" "user"
-    "status" "ssh-process" "ssl-process" "messages"
+    "status" "process" "ssh-process" "ssl-process"
+    "qfld" "messages"
     ;; parameters used internally and should be initialized
-    "string" "error" "done"))
+    "string" "error" "done" "imapp"))
 
 (mew-info-defun "mew-nntp2-" mew-nntp2-info-list)
 
 (defvar mew-nntp2-info-list-save-length 6)
-(defvar mew-nntp2-info-list-clean-length 14)
+(defvar mew-nntp2-info-list-clean-length 16)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -36,7 +37,7 @@
   '(("greeting"    ("20[01]" . "mode-reader")) ;; a broken server returns 200 even if post is allowed
     ("mode-reader" (t        . "authinfo"))
     ("authinfo"    ("381"    . "authpass"))
-    ("authpass"    ("281"    . "post") (t . "wpwd"))
+    ("authpass"    ("281"    . "next") (t . "wpwd"))
     ("post"        ("340"    . "post-post"))
     ("post-post"   ("240"    . "done"))
     ("quit"        (t        . "noop"))))
@@ -59,17 +60,37 @@
   (let ((user (mew-nntp2-get-user pnm)))
     (if user
 	(mew-nntp2-process-send-string pro "AUTHINFO USER %s" user)
-      (mew-nntp2-set-status pnm "post")
-      (mew-nntp2-command-post pro pnm))))
+      (mew-nntp2-set-status pnm "next")
+      (mew-nntp2-command-next pro pnm))))
 
 (defun mew-nntp2-command-authpass (pro pnm)
-  (let ((pass (mew-input-passwd "AUTHINFO: " (mew-nntp2-passtag pnm))))
+  (let ((pass (mew-nntp2-input-passwd "NNTP password: " pnm)))
     (mew-nntp2-process-send-string pro "AUTHINFO PASS %s" pass)))
 
 (defun mew-nntp2-command-wpwd (pro pnm)
-  (message "AUTHINFO password is wrong!")
   (mew-passwd-set-passwd (mew-nntp2-passtag pnm) nil)
-  (mew-nntp2-recover pro pnm "Wrong password for NNTP AUTH"))
+  (mew-nntp2-set-error pnm "NNTP password is wrong!")
+  (mew-nntp2-command-quit2 pro pnm))
+
+(defun mew-nntp2-command-next (pro pnm)
+  (let ((msgs (mew-nntp2-get-messages pnm))
+	(qfld (mew-nntp2-get-qfld pnm))
+ 	(case (mew-nntp2-get-case pnm))
+	msg)
+    (if msgs
+	(progn
+	  (setq msg (car msgs))
+	  (setq msgs (cdr msgs))
+	  (mew-queue-insert-file pnm mew-nntp2-info-list-save-length qfld msg)
+	  (mew-set-buffer-multibyte nil)
+	  (mew-info-clean-up pnm mew-nntp2-info-list-clean-length)
+ 	  (mew-nntp2-set-case pnm case) ;; override
+	  (mew-nntp2-set-messages pnm msgs)
+	  (set-process-buffer pro (current-buffer))
+	  (mew-nntp2-set-status pnm "post")
+	  (mew-nntp2-command-post pro pnm))
+      (mew-nntp2-set-status pnm "quit")
+      (mew-nntp2-command-quit pro pnm))))
 
 (defun mew-nntp2-command-post (pro pnm)
   (mew-nntp2-process-send-string pro "POST"))
@@ -85,46 +106,34 @@
 
 (defun mew-nntp2-command-done (pro pnm)
   (let ((fcc (mew-nntp2-get-fcc pnm))
+	(case (mew-nntp2-get-case pnm))
 	(back (mew-queue-backup (buffer-file-name) mew-queue-info-suffix))
 	(buf (process-buffer pro))
-	msgs)
+	imapp)
     ;; mew-folder-new-message may be slow if the folder contains
     ;; a lot of messages. So, let's Fcc in background.
-    (mew-net-fcc-message fcc back)
+    (setq imapp (mew-net-fcc-message case fcc back))
+    (mew-nntp2-set-imapp pnm imapp)
     (mew-nntp2-log pnm)
-    ;; killing buffer
     (set-process-buffer pro nil)
     (mew-remove-buffer buf)
-    (setq msgs (mew-nntp2-get-messages pnm))
-    (if (and msgs
-	     (mew-queue-get-next
-	      pnm msgs
-	      mew-nntp2-info-list-save-length
-	      'mew-nntp2-set-messages))
-	;; A file inserted. Now in msg's buffer
-	(progn
-	  (mew-info-clean-up pnm mew-nntp2-info-list-clean-length)
-	  (set-process-buffer pro (current-buffer))
-	  (mew-nntp2-set-status pnm "post")
-	  (mew-nntp2-command-post pro pnm))
-      (mew-nntp2-set-status pnm "quit")
-      (mew-nntp2-command-quit pro pnm))))
+    (mew-nntp2-set-status pro "next")
+    (mew-nntp2-command-next pro pnm)))
 
 (defun mew-nntp2-command-quit (pro pnm)
   (mew-nntp2-set-done pnm t)
   (mew-nntp2-process-send-string pro "QUIT"))
 
+(defun mew-nntp2-command-quit2 (pro pnm)
+  ;; error is set
+  (mew-nntp2-set-done pnm t)
+  (when (and (processp pro) (eq (process-status pro) 'open))
+    (mew-nntp2-set-status pnm "quit")
+    (mew-nntp2-process-send-string pro "QUIT")))
+
 (defun mew-nntp2-command-noop (pro pnm)
   ())
 
-(defun mew-nntp2-recover (pro pnm err)
-  (let ((case (mew-nntp2-get-case pnm)))
-    (mew-nntp2-log pnm err)
-    (mew-nntp2-queue case err)
-    (when (and (processp pro) (eq (process-status pro) 'open))
-      (mew-nntp2-set-status pnm "quit")
-      (mew-nntp2-command-quit pro pnm))))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Sub functions
@@ -147,7 +156,9 @@
 (defun mew-nntp2-process-send-string (pro &rest args)
   (let ((str (apply 'format args)))
     (mew-nntp2-debug "=SEND=" str)
-    (process-send-string pro (concat str mew-cs-eol))))
+    (if (and (processp pro) (eq (process-status pro) 'open))
+	(process-send-string pro (concat str mew-cs-eol))
+      (message "NNTP time out"))))
 
 (defun mew-nntp2-passtag (pnm)
   (let ((server (mew-nntp2-get-server pnm))
@@ -155,6 +166,14 @@
 	(user (mew-nntp2-get-user pnm)))
     (concat user "@" server ":" port)))
 
+(defun mew-nntp2-input-passwd (prompt pnm)
+  (let* ((tag (mew-nntp2-passtag pnm))
+         (pro (mew-nntp2-get-process pnm))
+         (pass (mew-input-passwd prompt tag)))
+    (unless (and (processp pro) (eq (process-status pro) 'open))
+      (mew-passwd-set-passwd tag nil))
+    pass))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Opening NNTP
@@ -162,7 +181,7 @@
 
 (defun mew-nntp2-open (pnm server port)
   (let ((sprt (mew-port-sanity-check port))
-	pro tm errmsg)
+	pro tm)
     (condition-case emsg
 	(progn
 	  (setq tm (mew-timer mew-nntp-timeout-time 'mew-nntp2-timeout))
@@ -172,14 +191,11 @@
 	  (mew-set-process-cs pro mew-cs-text-for-net mew-cs-text-for-net)
 	  (message "Connecting to the NNTP server...done"))
       (quit
-       (setq errmsg "Cannot connect to the NNTP server")
-       (mew-nntp2-set-error pnm errmsg)
        (setq pro nil)
-       (message "%s" errmsg))
+       (message "Cannot connect to the NNTP server"))
       (error
-       (setq errmsg (format "%s, %s" (nth 1 emsg) (nth 2 emsg)))
        (setq pro nil)
-       (message "%s" errmsg)))
+       (message "%s, %s" (nth 1 emsg) (nth 2 emsg))))
     (if tm (cancel-timer tm))
     pro))
 
@@ -191,83 +207,65 @@
 ;;; Launcher
 ;;;
 
-(defun mew-nntp2-send-message (case)
-  (let* ((server (mew-nntp-server case))
-	 (port (mew-nntp-port case))
-	 (pnm (mew-nntp2-info-name case))
-	 (sshsrv (mew-nntp-ssh-server case))
-	 (sslp (mew-nntp-ssl case))
-	 (sslport (mew-nntp-ssl-port case))
-	 process sshname sshpro sslname sslpro lport errmsg)
+(defun mew-nntp2-send-message (case qfld msgs)
+  (let ((server (mew-nntp-server case))
+	(port (mew-nntp-port case))
+	(pnm (mew-nntp2-info-name case))
+	(sshsrv (mew-nntp-ssh-server case))
+	(sslp (mew-nntp-ssl case))
+	(sslport (mew-nntp-ssl-port case))
+	process sshname sshpro sslname sslpro lport)
     (cond
-     ((null (mew-nntp2-get-newsgroups pnm))
-      (message "No newsgroups!"))
-     ((get-process pnm)
-      (setq errmsg "NNTP connection is locked")
-      (mew-nntp2-set-error pnm errmsg)
-      (mew-nntp2-queue case errmsg))
+     (sshsrv
+      (setq sshpro (mew-open-ssh-stream case server port sshsrv))
+      (when sshpro
+	(setq sshname (process-name sshpro))
+	(setq lport (mew-ssh-pnm-to-lport sshname))
+	(when lport
+	  (setq process (mew-nntp2-open pnm "localhost" lport)))))
+     (sslp
+      (setq sslpro (mew-open-ssl-stream server sslport))
+      (when sslpro
+	(setq sslname (process-name sslpro))
+	(setq lport (mew-ssl-pnm-to-lport sslname))
+	(when lport
+	  (setq process (mew-nntp2-open pnm "localhost" lport)))))
      (t
-      (cond
-       (sshsrv
-	(setq sshpro (mew-open-ssh-stream server port sshsrv))
-	(when sshpro
-	  (setq sshname (process-name sshpro))
-	  (setq lport (mew-ssh-pnm-to-lport sshname))
-	  (when lport
-	    (setq process (mew-nntp2-open pnm "localhost" lport)))))
-       (sslp
-	(setq sslpro (mew-open-ssl-stream server sslport))
-	(when sslpro
-	  (setq sslname (process-name sslpro))
-	  (setq lport (mew-ssl-pnm-to-lport sslname))
-	  (when lport
-	    (setq process (mew-nntp2-open pnm "localhost" lport)))))
-       (t
-	(setq process (mew-nntp2-open pnm server port))))
-      (if (null process)
-	  (progn
-	    (cond
-	     ((and sshsrv (null sshpro))
-	      (setq errmsg "Cannot create to the SSH connection"))
-	     ((and sslp (null sslpro))
-	      (setq errmsg "Cannot create to the SSL connection"))
-	     (t
-	      (setq errmsg "Cannot connect to the NNTP server")))
-	    (mew-nntp2-set-error pnm errmsg)
-	    (mew-nntp2-queue case errmsg))
-	(mew-info-clean-up pnm mew-nntp2-info-list-clean-length)
-	(mew-nntp2-set-case pnm case)
-	(mew-nntp2-set-server pnm server)
-	(mew-nntp2-set-port pnm port)
-	(mew-nntp2-set-ssh-server pnm sshsrv)
-	(mew-nntp2-set-ssh-process pnm sshpro)
-	(mew-nntp2-set-ssl-process pnm sslpro)
-	(mew-nntp2-set-user pnm (mew-nntp-user case))
-	(mew-nntp2-set-status pnm "greeting")
-	(set-process-buffer process (current-buffer))
-	(set-process-sentinel process 'mew-nntp2-sentinel)
-	(set-process-filter process 'mew-nntp2-filter)
-	(message "Posting in background..."))))))
+      (setq process (mew-nntp2-open pnm server port))))
+    (if (null process)
+	(cond
+	 ((and sshsrv (null sshpro))
+	  (message "Cannot create to the SSH connection"))
+	 ((and sslp (null sslpro))
+	  (message "Cannot create to the SSL connection"))
+	 (t
+	  (message "Cannot connect to the NNTP server")))
+      (mew-info-clean-up pnm mew-nntp2-info-list-clean-length)
+      (mew-nntp2-set-case pnm case)
+      (mew-nntp2-set-qfld pnm qfld)
+      (mew-nntp2-set-messages pnm msgs)
+      (mew-nntp2-set-server pnm server)
+      (mew-nntp2-set-port pnm port)
+      (mew-nntp2-set-ssh-server pnm sshsrv)
+      (mew-nntp2-set-ssh-process pnm sshpro)
+      (mew-nntp2-set-ssl-process pnm sslpro)
+      (mew-nntp2-set-user pnm (mew-nntp-user case))
+      (mew-nntp2-set-status pnm "greeting")
+      ;;
+      (set-process-buffer process nil)
+      (set-process-sentinel process 'mew-nntp2-sentinel)
+      (set-process-filter process 'mew-nntp2-filter)
+      (message "Posting in background..."))))
 
 (defun mew-nntp2-flush-queue (case &optional qfld)
-  (let* ((pnm (mew-nntp2-info-name case))
-	 msgs)
-    (unless qfld
-      (setq qfld (mew-postq-folder case)))
-    (if (get-process pnm)
-	(message "NNTP connection for %s is locked" qfld)
+  (let (msgs)
+    (unless qfld (setq qfld (mew-postq-folder case)))
+    (setq msgs (mew-folder-messages qfld))
+    (when msgs
       (mew-summary-folder-cache-clean qfld)
-      (setq msgs (directory-files (mew-expand-folder qfld)
-				  'full mew-regex-message-files))
-      (when (and msgs
-		 (mew-queue-get-next
-		  pnm msgs
-		  mew-nntp2-info-list-save-length
-		  'mew-nntp2-set-messages))
-	;; A file inserted. Now in msg's buffer
-	(run-hooks 'mew-nntp2-flush-hook)
-	(message "Flushing %s..." qfld)
-	(mew-nntp2-send-message case)))))
+      (run-hooks 'mew-nntp2-flush-hook)
+      (message "Flushing %s..." qfld)
+      (mew-nntp2-send-message case qfld msgs))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -283,43 +281,67 @@
 
 (defun mew-nntp2-filter (process string)
   (let* ((pnm (process-name process))
+	 (case (mew-nntp2-get-case pnm))
 	 (status (mew-nntp2-get-status pnm))
 	 (str (concat (mew-nntp2-get-string pnm) string))
+	 (buf (process-buffer process))
 	 next func code)
     (mew-nntp2-debug (upcase status) string)
-    (mew-filter
-     ;; Process's buffer
-     (mew-nntp2-set-string pnm str)
-     (cond
-      ((and (string-match "\n$" str)
-	    (string-match "^\\([0-9][0-9][0-9]\\) " str))
-       (setq code (match-string 1 str))
-       (setq next (mew-nntp2-fsm-next status code))
-       (cond
-	(next
-	 (mew-nntp2-set-status pnm next)
-	 (setq func (intern-soft (concat "mew-nntp2-command-" next)))
-	 (and func (funcall func process pnm))
-	 (mew-nntp2-set-string pnm nil))
-	(t
-	 (if (string-match "^pwd-" status)
-	     (message "Password is wrong"))
-	 (mew-nntp2-recover process pnm str))))
-      (t ()))))) ;; stay
+    (if (and buf (get-buffer buf)) (set-buffer buf))
+    ;; NNTP server's strings should be short enough.
+    (mew-nntp2-set-string pnm str)
+    (cond
+     ((and (string-match "\n$" str)
+	   (string-match "^\\([0-9][0-9][0-9]\\) " str))
+      (setq code (match-string 1 str))
+      (setq next (mew-nntp2-fsm-next status code))
+      (cond
+       (next
+	(mew-nntp2-set-status pnm next)
+	(setq func (intern-soft (concat "mew-nntp2-command-" next)))
+	(and func (funcall func process pnm))
+	(mew-nntp2-set-string pnm nil))
+       (t
+	(if (string-match "^pwd-" status)
+	    (mew-nntp2-set-error pnm "NNTP password is wrong!")
+ 	  (if (string-match "\n$" str)
+ 	      (setq str (substring str 0 (1- (length str)))))
+ 	  (unless (string-match "[.!]$" str)
+ 	    (setq str (concat str "."))))
+	(mew-nntp2-queue case str)
+	(mew-nntp2-set-error pnm str)
+	(mew-nntp2-command-quit2 process pnm))))
+     (t ())))) ;; stay
 
 (defun mew-nntp2-sentinel (process event)
   (let* ((pnm (process-name process))
+	 (buf (process-buffer process))
+	 (qfld (mew-nntp2-get-qfld pnm))
+	 (case (mew-nntp2-get-case pnm))
 	 (done (mew-nntp2-get-done pnm))
+	 (imapp (mew-nntp2-get-imapp pnm))
+	 (error (mew-nntp2-get-error pnm))
 	 (sshpro (mew-nntp2-get-ssh-process pnm))
 	 (sslpro (mew-nntp2-get-ssl-process pnm)))
     (mew-nntp2-debug "NNTP SENTINEL" event)
+    (cond
+     (error
+      (message "%s  This mail has been queued to %s" error qfld)
+      (mew-nntp2-log pnm error))
+     (done
+      (message "Posting in background...done"))
+     (t 
+      (if (null buf)
+	  (message "NNTP connection is lost")
+	(set-buffer buf)
+	(mew-nntp2-queue case "NNTP connection is lost"))))
     (mew-info-clean-up pnm)
     (if (and (processp sshpro) (not mew-ssh-keep-connection))
 	(process-send-string sshpro "exit\n"))
     (if (and (processp sslpro) (not mew-ssl-keep-connection))
 	(delete-process sslpro))
-    (if done (message "Posting in background...done"))
-    (run-hooks 'mew-nntp2-sentinel-hook)))
+    (run-hooks 'mew-nntp2-sentinel-hook)
+    (if imapp (mew-imap2-fcc case))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -336,7 +358,8 @@
     (mew-local-folder-check qfld)
     (setq file-info (mew-queue-enqueue work qfld))
     (setq file (nth 0 file-info) info (nth 1 file-info))
-    (setq nname (mew-concat-folder qfld (file-name-nondirectory file)))
+    (setq file (file-name-nondirectory file))
+    (setq nname (mew-concat-folder qfld file))
     (if (mew-draft-p)
 	(mew-nntp2-set-case pnm (mew-tinfo-get-case)))
     ;;
@@ -351,7 +374,8 @@
     (mew-remove-buffer (current-buffer))
     (message "%s has been queued to %s (%s)"
 	     oname nname (or (mew-nntp2-get-error pnm) err))
-    (mew-touch-folder qfld)))
+    (mew-touch-folder qfld)
+    file))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
