@@ -470,14 +470,19 @@ Return t if exists or created. Otherwise, return nil."
      ((mew-folder-imapp folder)
       (let* ((case mew-inherit-refile-case)
 	     (mailboxes (mew-imap-folder-alist case)))
-	(if (or (assoc folder mailboxes)
-		(assoc (mew-imap-file-name-as-directory folder case) mailboxes))
-	    t
-	  (if (and (y-or-n-p (format "%s does not exist. Create it? " folder))
-		   (mew-imap-folder-insert case folder))
-	      (progn
-		(message "%s will be created" folder)
-		t)))))
+	(cond
+	 ((or (assoc folder mailboxes)
+	      (assoc (mew-imap-file-name-as-directory folder case) mailboxes))
+	  t)
+	 ;; This is very ad-hoc but no right way...
+	 ((and (string= (mew-imap-separator case) ".")
+	       (string-match mew-path-separator folder))
+	  (message "%s should not contain \"%s\"" folder mew-path-separator)
+	  nil)
+	 ((and (y-or-n-p (format "%s does not exist. Create it? " folder))
+	       (mew-imap-folder-insert case folder))
+	  (message "%s will be created" folder)
+	  t))))
      (t ;; local
       (let ((absdir (mew-expand-folder folder))  ;; /home/Mail/foo
 	    (create-it force-to-create))
@@ -598,8 +603,8 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
 
 (defun mew-summary-refile-remove-body ()
   (save-excursion
-    (mew-summary-goto-body 'after)
-    (when (looking-at mew-regex-override-body)
+    (when (and (mew-summary-goto-body 'after)
+	       (looking-at mew-regex-override-body))
       (mew-elet
        (delete-region (point) (match-end 0))))))
 
