@@ -16,20 +16,21 @@
   '(;; parameters to be saved
     "raw-header" "recipients" "orig-recipients"
     "bcc" "dcc" "fcc" "msgid" "logtime"
-    "case" ;; save for re-edit, not for sending
+    "case" ;; save for re-edit
     ;; parameters used internally
     "server" "port" "ssh-server"
     "user" "auth-list"
     "helo-domain" 
-    "status" "ssh-process" "ssl-process" "messages"
+    "status" "process" "ssh-process" "ssl-process"
+    "qfld" "messages"
     ;; parameters used internally and should be initialized
     "string" "error" "auth-selected" "timer" "cont" "from" "sender"
-    "done"))
+    "done" "imapp"))
 
 (mew-info-defun "mew-smtp-" mew-smtp-info-list)
 
 (defvar mew-smtp-info-list-save-length 9)
-(defvar mew-smtp-info-list-clean-length 19)
+(defvar mew-smtp-info-list-clean-length 21)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -40,14 +41,14 @@
   '(("greeting"      ("220" . "ehlo"))
     ("ehlo"          ("250" . "auth") (t . "helo"))
     ;;
-    ("auth"          ("250" . "mail-from"))
+    ("auth"          ("250" . "next"))
     ("auth-cram-md5" ("334" . "pwd-cram-md5") (t . "wpwd"))
-    ("pwd-cram-md5"  ("235" . "mail-from") (t . "wpwd"))
+    ("pwd-cram-md5"  ("235" . "next") (t . "wpwd"))
     ("auth-login"    ("334" . "pwd-login") (t . "wpwd"))
-    ("pwd-login"     ("235" . "mail-from") (t . "wpwd"))
-    ("auth-plain"    ("235" . "mail-from") (t . "wpwd"))
+    ("pwd-login"     ("235" . "next") (t . "wpwd"))
+    ("auth-plain"    ("235" . "next") (t . "wpwd"))
     ;;
-    ("helo"          ("250" . "mail-from"))
+    ("helo"          ("250" . "next"))
     ("mail-from"     ("250" . "rcpt-to"))
     ("rcpt-to"       ("250" . "data"))
     ("data"          ("354" . "content"))
@@ -87,16 +88,37 @@
 	    (funcall func pro pnm))
 	(mew-smtp-debug "<AUTH>" "No preferred SMTP AUTH.\n")
 	;; xxx how should we act?
-	(mew-smtp-set-status pnm "mail-from")
-	(mew-smtp-command-mail-from pro pnm))))
+	(mew-smtp-set-status pnm "next")
+	(mew-smtp-command-next pro pnm))))
    (t
-    (mew-smtp-set-status pnm "mail-from")
-    (mew-smtp-command-mail-from pro pnm))))
+    (mew-smtp-set-status pnm "next")
+    (mew-smtp-command-next pro pnm))))
 
 (defun mew-smtp-command-wpwd (pro pnm)
   (let ((auth (mew-smtp-get-auth-selected pnm)))
-    (mew-passwd-set-passwd (mew-smtp-passtag pnm auth) nil)
-    (mew-smtp-recover pro pnm (format "Wrong password for %s" auth))))
+    (mew-passwd-set-passwd (mew-smtp-passtag pnm) nil)
+    (mew-smtp-set-error pnm (format "SMTP %s password is wrong!" auth))
+    (mew-smtp-command-quit2 pro pnm)))
+
+(defun mew-smtp-command-next (pro pnm)
+  (let ((msgs (mew-smtp-get-messages pnm))
+	(qfld (mew-smtp-get-qfld pnm))
+	(case (mew-smtp-get-case pnm))
+	msg)
+    (if msgs
+	(progn
+	  (setq msg (car msgs))
+	  (setq msgs (cdr msgs))
+	  (mew-queue-insert-file pnm mew-smtp-info-list-save-length qfld msg)
+	  (mew-set-buffer-multibyte nil)
+	  (mew-info-clean-up pnm mew-smtp-info-list-clean-length)
+	  (mew-smtp-set-case pnm case) ;; override
+	  (mew-smtp-set-messages pnm msgs)
+	  (set-process-buffer pro (current-buffer))
+	  (mew-smtp-set-status pnm "mail-from")
+	  (mew-smtp-command-mail-from pro pnm))
+      (mew-smtp-set-status pnm "quit")
+      (mew-smtp-command-quit pro pnm))))
 
 (defun mew-smtp-command-mail-from (pro pnm)
   (widen)
@@ -153,7 +175,8 @@
       ;;
       (while (and (< cont (point-max)) (not (input-pending-p)) (< i N))
         (let ((next (min (point-max) (+ cont inc))))
-          (process-send-region pro cont next) ;; xxx
+	  (if (and (processp pro) (eq (process-status pro) 'open))
+	      (process-send-region pro cont next))
           (setq cont next)
           (setq i (1+ i))))
       (mew-smtp-set-cont pnm cont)
@@ -170,48 +193,36 @@
 
 (defun mew-smtp-command-done (pro pnm)
   (let ((fcc (mew-smtp-get-fcc pnm))
+	(case (mew-smtp-get-case pnm))
 	(back (mew-queue-backup (buffer-file-name) mew-queue-info-suffix))
 	(buf (process-buffer pro))
-	msgs)
+	imapp)
     ;; mew-folder-new-message may be slow if the folder contains
     ;; a lot of messages. So, let's Fcc in background.
-    (mew-net-fcc-message fcc back)
+    (setq imapp (mew-net-fcc-message case fcc back))
+    (mew-smtp-set-imapp pnm imapp)
     (mew-smtp-log pnm)
     (if (mew-smtp-get-bcc pnm)
 	(mew-smtp-bcc pro pnm back)
-      ;; killing buffer
       (set-process-buffer pro nil)
       (mew-remove-buffer buf)
-      (setq msgs (mew-smtp-get-messages pnm))
-      (if (and msgs
-	       (mew-queue-get-next
-		pnm msgs
-		mew-smtp-info-list-save-length
-		'mew-smtp-set-messages))
-	  ;; A file inserted. Now in msg's buffer
-	  (progn
-	    (mew-info-clean-up pnm mew-smtp-info-list-clean-length)
-	    (set-process-buffer pro (current-buffer))
-	    (mew-smtp-set-status pnm "mail-from")
-	    (mew-smtp-command-mail-from pro pnm))
-	(mew-smtp-set-status pnm "quit")
-	(mew-smtp-command-quit pro pnm)))))
+      (mew-smtp-set-status pro "next")
+      (mew-smtp-command-next pro pnm))))
 
 (defun mew-smtp-command-quit (pro pnm)
   (mew-smtp-set-done pnm t)
   (mew-smtp-process-send-string pro "QUIT"))
 
+(defun mew-smtp-command-quit2 (pro pnm)
+  ;; error is set
+  (mew-smtp-set-done pnm t)
+  (when (and (processp pro) (eq (process-status pro) 'open))
+    (mew-smtp-set-status pnm "quit")
+    (mew-smtp-process-send-string pro "QUIT")))
+
 (defun mew-smtp-command-noop (pro pnm)
   ())
 
-(defun mew-smtp-recover (pro pnm err)
-  (let ((case (mew-smtp-get-case pnm)))
-    (mew-smtp-log pnm err)
-    (mew-smtp-queue case err)
-    (when (and (processp pro) (eq (process-status pro) 'open))
-      (mew-smtp-set-status pnm "quit")
-      (mew-smtp-command-quit pro pnm))))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; AUTH
@@ -237,7 +248,7 @@
 	challenge passwd cram-md5)
     (if (string-match " \\([a-zA-Z0-9+/]+=*\\)" str) ;; xxx
 	(setq challenge (match-string 1 str)))
-    (setq passwd (mew-smtp-input-passwd "CRAM-MD5 password: " pnm "cram-md5"))
+    (setq passwd (mew-smtp-input-passwd "SMTP CRAM-MD5 password: " pnm))
     (setq cram-md5 (mew-cram-md5 user passwd challenge))
     (mew-smtp-process-send-string pro cram-md5)))
 
@@ -248,12 +259,12 @@
     (mew-smtp-set-status pnm "auth-login")))
 
 (defun mew-smtp-command-pwd-login (pro pnm)
-  (let* ((passwd (mew-smtp-input-passwd "LOGIN password: " pnm "login"))
+  (let* ((passwd (mew-smtp-input-passwd "SMTP LOGIN password: " pnm))
 	 (epasswd (mew-base64-encode-string passwd)))
     (mew-smtp-process-send-string pro epasswd)))
 
 (defun mew-smtp-command-auth-plain (pro pnm)
-  (let* ((passwd (mew-smtp-input-passwd "PLAIN password: " pnm "plain"))
+  (let* ((passwd (mew-smtp-input-passwd "SMTP PLAIN password: " pnm))
 	 (user (mew-smtp-get-user pnm))
 	 (plain (mew-base64-encode-string (format "\0%s\0%s" user passwd))))
     (mew-smtp-process-send-string pro "AUTH PLAIN %s" plain)
@@ -284,17 +295,23 @@
 (defun mew-smtp-process-send-string (pro &rest args)
   (let ((str (apply 'format args)))
     (mew-smtp-debug "=SEND=" str)
-    (process-send-string pro (concat str mew-cs-eol))))
+    (if (and (processp pro) (eq (process-status pro) 'open))
+	(process-send-string pro (concat str mew-cs-eol))
+      (message "SMTP time out"))))
 
-(defun mew-smtp-passtag (pnm auth)
-  (concat (downcase auth)
-	  "/" (mew-smtp-get-user pnm)
+(defun mew-smtp-passtag (pnm)
+  (concat (mew-smtp-get-user pnm)
 	  "@" (mew-smtp-get-server pnm)
 	  ":" (mew-smtp-get-port pnm)))
 
-(defun mew-smtp-input-passwd (prompt pnm auth)
-  (let ((tag (mew-smtp-passtag pnm auth)))
-    (mew-input-passwd prompt tag)))
+(defun mew-smtp-input-passwd (prompt pnm)
+  (let ((tag (mew-smtp-passtag pnm))
+	(pro (mew-smtp-get-process pnm))
+	pass)
+    (setq pass (mew-input-passwd prompt tag))
+    (unless (and (processp pro) (eq (process-status pro) 'open))
+      (mew-passwd-set-passwd tag nil))
+    pass))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -303,7 +320,7 @@
 
 (defun mew-smtp-open (pnm server port)
   (let ((sprt (mew-port-sanity-check port))
-	pro tm errmsg)
+	pro tm)
     (condition-case emsg
 	(progn
 	  (setq tm (mew-timer mew-smtp-timeout-time 'mew-smtp-timeout))
@@ -313,15 +330,11 @@
 	  (mew-set-process-cs pro mew-cs-text-for-net mew-cs-text-for-net)
 	  (message "Connecting to the SMTP server...done"))
       (quit
-       (setq errmsg "Cannot connect to the SMTP server")
-       (mew-smtp-set-error pnm errmsg)
        (setq pro nil)
-       (message "%s" errmsg))
+       (message "Cannot connect to the SMTP server"))
       (error
-       (setq errmsg (format "%s, %s" (nth 1 emsg) (nth 2 emsg)))
-       (mew-smtp-set-error pnm errmsg)
        (setq pro nil)
-       (message "%s" errmsg)))
+       (message "%s, %s" (nth 1 emsg) (nth 2 emsg))))
     (if tm (cancel-timer tm))
     pro))
 
@@ -333,85 +346,68 @@
 ;;; Launcher
 ;;;
 
-(defun mew-smtp-send-message (case)
-  (let* ((server (mew-smtp-server case))
-	 (port (mew-smtp-port case))
-	 (pnm (mew-smtp-info-name case))
-	 (sshsrv (mew-smtp-ssh-server case))
-	 (sslp (mew-smtp-ssl case))
-	 (sslport (mew-smtp-ssl-port case))
-	 process sshname sshpro sslname sslpro lport errmsg)
+(defun mew-smtp-send-message (case qfld msgs)
+  (let ((server (mew-smtp-server case))
+	(port (mew-smtp-port case))
+	(pnm (mew-smtp-info-name case))
+	(sshsrv (mew-smtp-ssh-server case))
+	(sslp (mew-smtp-ssl case))
+	(sslport (mew-smtp-ssl-port case))
+	process sshname sshpro sslname sslpro lport)
     (cond
-     ((null (mew-smtp-get-recipients pnm))
-      (message "No recipient!"))
-     ((get-process pnm)
-      (setq errmsg "SMTP connection is locked")
-      (mew-smtp-set-error pnm errmsg)
-      (mew-smtp-queue case errmsg))
+     (sshsrv
+      (setq sshpro (mew-open-ssh-stream case server port sshsrv))
+      (when sshpro
+	(setq sshname (process-name sshpro))
+	(setq lport (mew-ssh-pnm-to-lport sshname))
+	(when lport
+	  (setq process (mew-smtp-open pnm "localhost" lport)))))
+     (sslp
+      (setq sslpro (mew-open-ssl-stream server sslport))
+      (when sslpro
+	(setq sslname (process-name sslpro))
+	(setq lport (mew-ssl-pnm-to-lport sslname))
+	(when lport
+	  (setq process (mew-smtp-open pnm "localhost" lport)))))
      (t
-      (cond
-       (sshsrv
-	(setq sshpro (mew-open-ssh-stream server port sshsrv))
-	(when sshpro
-	  (setq sshname (process-name sshpro))
-	  (setq lport (mew-ssh-pnm-to-lport sshname))
-	  (when lport
-	    (setq process (mew-smtp-open pnm "localhost" lport)))))
-       (sslp
-	(setq sslpro (mew-open-ssl-stream server sslport))
-	(when sslpro
-	  (setq sslname (process-name sslpro))
-	  (setq lport (mew-ssl-pnm-to-lport sslname))
-	  (when lport
-	    (setq process (mew-smtp-open pnm "localhost" lport)))))
-       (t
-	(setq process (mew-smtp-open pnm server port))))
-      (if (null process)
-	  (progn
-	    (cond
-	     ((and sshsrv (null sshpro))
-	      (setq errmsg "Cannot create to the SSH connection"))
-	     ((and sslp (null sslpro))
-	      (setq errmsg "Cannot create to the SSL connection"))
-	     (t
-	      (setq errmsg "Cannot connect to the SMTP server")))
-	    (mew-smtp-set-error pnm errmsg)
-	    (mew-smtp-queue case errmsg))
-	(mew-info-clean-up pnm mew-smtp-info-list-clean-length)
-	(mew-smtp-set-case pnm case)
-	(mew-smtp-set-server pnm server)
-	(mew-smtp-set-port pnm port)
-	(mew-smtp-set-ssh-server pnm sshsrv)
-	(mew-smtp-set-ssh-process pnm sshpro)
-	(mew-smtp-set-ssl-process pnm sslpro)
-	(mew-smtp-set-helo-domain pnm (mew-smtp-helo-domain case))
-	(mew-smtp-set-user pnm (mew-smtp-user case))
-	(mew-smtp-set-auth-list pnm (mew-smtp-auth-list case))
-	(mew-smtp-set-status pnm "greeting")
-	(set-process-buffer process (current-buffer))
-	(set-process-sentinel process 'mew-smtp-sentinel)
-	(set-process-filter process 'mew-smtp-filter)
-	(message "Sending in background..."))))))
+      (setq process (mew-smtp-open pnm server port))))
+    (if (null process)
+	(cond
+	 ((and sshsrv (null sshpro))
+	  (message "Cannot create to the SSH connection"))
+	 ((and sslp (null sslpro))
+	  (message "Cannot create to the SSL connection"))
+	 (t
+	  (message "Cannot connect to the SMTP server")))
+      (mew-info-clean-up pnm mew-smtp-info-list-clean-length)
+      (mew-smtp-set-case pnm case)
+      (mew-smtp-set-qfld pnm qfld)
+      (mew-smtp-set-messages pnm msgs)
+      (mew-smtp-set-server pnm server)
+      (mew-smtp-set-port pnm port)
+      (mew-smtp-set-process pnm process)
+      (mew-smtp-set-ssh-server pnm sshsrv)
+      (mew-smtp-set-ssh-process pnm sshpro)
+      (mew-smtp-set-ssl-process pnm sslpro)
+      (mew-smtp-set-helo-domain pnm (mew-smtp-helo-domain case))
+      (mew-smtp-set-user pnm (mew-smtp-user case))
+      (mew-smtp-set-auth-list pnm (mew-smtp-auth-list case))
+      (mew-smtp-set-status pnm "greeting")
+      ;;
+      (set-process-buffer process nil)
+      (set-process-sentinel process 'mew-smtp-sentinel)
+      (set-process-filter process 'mew-smtp-filter)
+      (message "Sending in background..."))))
 
 (defun mew-smtp-flush-queue (case &optional qfld)
-  (let* ((pnm (mew-smtp-info-name case))
-	 msgs)
-    (unless qfld
-      (setq qfld (mew-queue-folder case)))
-    (if (get-process pnm)
-	(message "SMTP connection for %s is locked" qfld)
+  (let (msgs)
+    (unless qfld (setq qfld (mew-queue-folder case)))
+    (setq msgs (mew-folder-messages qfld))
+    (when msgs
       (mew-summary-folder-cache-clean qfld)
-      (setq msgs (directory-files (mew-expand-folder qfld)
-				  'full mew-regex-message-files))
-      (when (and msgs
-		 (mew-queue-get-next
-		  pnm msgs
-		  mew-smtp-info-list-save-length
-		  'mew-smtp-set-messages))
-	;; A file inserted. Now in msg's buffer
-	(run-hooks 'mew-smtp-flush-hook)
-	(message "Flushing %s..." qfld)
-	(mew-smtp-send-message case)))))
+      (run-hooks 'mew-smtp-flush-hook)
+      (message "Flushing %s..." qfld)
+      (mew-smtp-send-message case qfld msgs))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -427,43 +423,67 @@
 
 (defun mew-smtp-filter (process string)
   (let* ((pnm (process-name process))
+	 (case (mew-smtp-get-case pnm))
 	 (status (mew-smtp-get-status pnm))
 	 (str (concat (mew-smtp-get-string pnm) string))
+	 (buf (process-buffer process))
 	 next func code)
     (mew-smtp-debug (upcase status) string)
-    (mew-filter
-     ;; SMTP server's strings should be short enough.
-     (mew-smtp-set-string pnm str)
-     (cond
-      ((and (string-match "\n$" str)
-	    (string-match "^\\([1-5][0-7][0-9]\\) " str))
-       (setq code (match-string 1 str))
-       (setq next (mew-smtp-fsm-next status code))
-       (cond
-	(next
-	 (mew-smtp-set-status pnm next)
-	 (setq func (intern-soft (concat "mew-smtp-command-" next)))
-	 (and func (funcall func process pnm))
-	 (mew-smtp-set-string pnm nil))
-	(t
-	 (if (string-match "^pwd-" status)
-	     (message "Password is wrong"))
-	 (mew-smtp-recover process pnm str))))
-      (t ()))))) ;; stay
+    (if (and buf (get-buffer buf)) (set-buffer buf))
+    ;; SMTP server's strings should be short enough.
+    (mew-smtp-set-string pnm str)
+    (cond
+     ((and (string-match "\n$" str)
+	   (string-match "^\\([1-5][0-7][0-9]\\) " str))
+      (setq code (match-string 1 str))
+      (setq next (mew-smtp-fsm-next status code))
+      (cond
+       (next
+	(mew-smtp-set-status pnm next)
+	(setq func (intern-soft (concat "mew-smtp-command-" next)))
+	(and func (funcall func process pnm))
+	(mew-smtp-set-string pnm nil))
+       (t
+	(if (string-match "^pwd-" status)
+	    (mew-smtp-set-error pnm "SMTP password is wrong!")
+	  (if (string-match "\n$" str)
+	      (setq str (substring str 0 (1- (length str)))))
+	  (unless (string-match "[.!]$" str)
+	    (setq str (concat str "."))))
+	(mew-smtp-queue case str)
+	(mew-smtp-set-error pnm str)
+	(mew-smtp-command-quit2 process pnm))))
+     (t ())))) ;; stay
 
 (defun mew-smtp-sentinel (process event)
   (let* ((pnm (process-name process))
+	 (buf (process-buffer process))
+	 (qfld (mew-smtp-get-qfld pnm))
+	 (case (mew-smtp-get-case pnm))
 	 (done (mew-smtp-get-done pnm))
+	 (imapp (mew-smtp-get-imapp pnm))
+	 (error (mew-smtp-get-error pnm))
 	 (sshpro (mew-smtp-get-ssh-process pnm))
 	 (sslpro (mew-smtp-get-ssl-process pnm)))
     (mew-smtp-debug "SMTP SENTINEL" event)
+    (cond
+     (error
+      (message "%s  This mail has been queued to %s" error qfld)
+      (mew-smtp-log pnm error))
+     (done
+      (message "Sending in background...done"))
+     (t
+      (if (null buf)
+	  (message "SMTP connection is lost")
+	(set-buffer buf)
+	(mew-smtp-queue case "SMTP connection is lost"))))
     (mew-info-clean-up pnm)
     (if (and (processp sshpro) (not mew-ssh-keep-connection))
 	(process-send-string sshpro "exit\n"))
     (if (and (processp sslpro) (not mew-ssl-keep-connection))
 	(delete-process sslpro))
-    (if done (message "Sending in background...done"))
-    (run-hooks 'mew-smtp-sentinel-hook)))
+    (run-hooks 'mew-smtp-sentinel-hook)
+    (if imapp (mew-imap2-fcc case))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -480,7 +500,8 @@
     (mew-local-folder-check qfld)
     (setq file-info (mew-queue-enqueue work qfld))
     (setq file (nth 0 file-info) info (nth 1 file-info))
-    (setq nname (mew-concat-folder qfld (file-name-nondirectory file)))
+    (setq file (file-name-nondirectory file))
+    (setq nname (mew-concat-folder qfld file))
     (if (mew-draft-p)
 	(mew-smtp-set-case pnm (mew-tinfo-get-case)))
     ;;
@@ -496,7 +517,8 @@
     (mew-remove-buffer (current-buffer))
     (message "%s has been queued to %s (%s)"
 	     oname nname (or (mew-smtp-get-error pnm) err))
-    (mew-touch-folder qfld)))
+    (mew-touch-folder qfld)
+    file))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -537,6 +559,7 @@
      (mew-encode-make-header)
      (mew-encode-save-draft)
      (mew-overlay-delete-buffer)
+     (mew-set-buffer-multibyte nil)
      (mew-info-clean-up pnm mew-smtp-info-list-clean-length)
      (set-process-buffer pro (current-buffer))
      (mew-smtp-set-status pnm "mail-from")
