@@ -308,7 +308,7 @@ sorted. But if you do so, \"\\<mew-summary-mode-map>\\[mew-summary-ls]\" + 'sync
 			 (point-min) 'mew-sort-orig)))
 	      ;; 'mew-sort-orign may start with bob.
 	      (if (null orig)
-		  (push-mark (point-max) t t)
+		  (mew-push-mark)
 		(save-excursion
 		  (goto-char orig)
 		  (beginning-of-line)
@@ -319,7 +319,7 @@ sorted. But if you do so, \"\\<mew-summary-mode-map>\\[mew-summary-ls]\" + 'sync
 		 ;; it.
 		 (remove-text-properties
 		  orig (+ orig (window-width)) '(mew-sort-orig nil)))
-		(push-mark orig t t)))
+		(mew-push-mark)))
 	    (mew-summary-folder-cache-save)
 	    (set-buffer-modified-p nil)))
 	  ;;
