@@ -226,13 +226,15 @@ and retern (beg . end)."
 	target prev pos)
     (if (boundp 'mode-line-position)
 	(progn
-	  (make-local-variable 'mode-line-position)
+	  (make-local-variable 'mode-line-position) ;; Emacs 21.3.50
 	  (setq mode-line-position
 		(copy-sequence (default-value 'mode-line-position)))
 	  (setq prev mode-line-position))
       (setq mode-line-format (copy-sequence (default-value 'mode-line-format)))
       (setq prev mode-line-format))
-    (setq target (or (rassoc tgt prev) (car (member tgt prev))))
+    (setq target (or (rassoc (list tgt) prev) ;; Emacs 21.3.50
+		     (rassoc tgt prev)
+		     (car (member tgt prev))))
     (when target
       (setq pos (- (length prev) (length (member target prev))))
       (setcar (nthcdr pos prev) mew-mode-line-format))
@@ -491,6 +493,9 @@ The keys that are defined for both Summary mode and Virtual mode are:
 	'Summary & Message mode'. If you choose 'Summary mode only',
 	you can quickly put the delete marks since the next message is
 	not displayed.
+\\[mew-summary-toggle-header-veil]	If 'mew-use-header-veil' is non-nil, field lines of To: and Cc:
+	over 'mew-header-veil-count' are covered with invisible veils.
+	This commands toggles visibility of these lines.
 
 \\[mew-summary-jump-to-draft-buffer]	Jump to one of drafts if exists.
 
@@ -540,6 +545,11 @@ The keys that are defined for both Summary mode and Virtual mode are:
 	will add personal information.  Otherwise, it will add an
 	alias.
 
+\\[mew-summary-alias-edit]	Editing the auto alias file which contains a list of alias-address
+	pairs. Remove unnecessary entries and save the file by
+	'\\[save-buffer]'. After saving, the modification is automatically
+	reflected.
+
 \\[mew-summary-kill-subprocess]	Kill a process in Summary mode.
 	Sometime a process accidentally remains in Summary mode.  In
 	this situation, you cannot execute '\\[mew-summary-retrieve]',
@@ -660,7 +670,7 @@ folder.
 	to picked messages from multiple folders. Enter a virtual
 	folder name, comma-separated folders, and pick pattern.
 
-\\[mew-summary-mark-virtual]	Making Virtaul mode for messages marked with '*'.
+\\[mew-summary-mark-virtual]	Making Virtual mode for messages marked with '*'.
 
 \\[mew-summary-sort]	Sort messages and list them up again.
 	If called with '\\[universal-argument]', sort the region.
@@ -668,10 +678,18 @@ folder.
 	or the region. If this command is used in a remote folder,
 	local cache messages are sorted.
 
+\\[mew-summary-delete-folder]	Delete this folder.
+\\[mew-summary-rename-folder]	Rename this folder.
+
 \\[mew-summary-make-thread]	This command makes threads for the
 	Summary mode as Virtual mode, then the cursor jump onto the current
 	message in the Virtual mode. If a corresponding Virtual mode exists,
 	this command just visits the Virtual mode.
+\\[mew-summary-regexp-make-thread]	Make threads for messages matched to a specified regular expression.
+
+\\[mew-summary-learn-spam]	Learn that this message is a spam.
+
+\\[mew-summary-learn-ham]	Learn that this message is a ham.
 
 Candidates of RANGE for local folders are as follows:
 	all
