@@ -139,6 +139,7 @@ If nodisplay is non-nil, displays the cached message."
 (defun mew-summary-cursor-postscript ()
   (mew-summary-mode-line)
   (mew-summary-recenter)
+  (mew-thread-move-cursor)
   (mew-highlight-cursor-line)
   (set-buffer-modified-p nil))
 
@@ -648,25 +649,33 @@ over the window. Type '\\[mew-summary-prev-page]' to see them when a message is
 ;;; Cursor motion
 ;;;
 
-(defun mew-summary-next-line ()
-  (interactive)
-  (beginning-of-line)
-  (forward-line)
-  (while (or (looking-at mew-regex-thread-separator)
-	     (eq (get-text-property (point) 'invisible) t))
-    (forward-line))
-  (if (and (mew-thread-p) mew-use-thread-cursor)
-      (mew-thread-move-cursor)))
+(defun mew-summary-next-line (&optional arg)
+  (interactive "p")
+  (or arg (setq arg 1))
+  (if (< arg 0)
+      (mew-summary-previous-line (- arg))
+    (let ((col (current-column)))
+      (while (> arg 0)
+	(forward-line)
+	(while (or (looking-at mew-regex-thread-separator)
+		   (eq (get-text-property (point) 'invisible) t))
+	  (forward-line))
+	(setq arg (1- arg)))
+      (move-to-column col))))
 
-(defun mew-summary-previous-line ()
-  (interactive)
-  (beginning-of-line)
-  (forward-line -1)
-  (while (or (looking-at mew-regex-thread-separator)
-	     (eq (get-text-property (point) 'invisible) t))
-    (forward-line -1))
-  (if (and (mew-thread-p) mew-use-thread-cursor)
-      (mew-thread-move-cursor)))
+(defun mew-summary-previous-line (&optional arg)
+  (interactive "p")
+  (or arg (setq arg 1))
+  (if (< arg 0)
+      (mew-summary-next-line (- arg))
+    (let ((col (current-column)))
+      (while (> arg 0)
+	(forward-line -1)
+	(while (or (looking-at mew-regex-thread-separator)
+		   (eq (get-text-property (point) 'invisible) t))
+	  (forward-line -1))
+	(setq arg (1- arg)))
+      (move-to-column col))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
