@@ -16,8 +16,7 @@
   "Common procedure to prepare a draft."
   `(progn
      (unwind-protect
-	 (let ((find-file-hooks nil)
-	       (inhibit-quit t))
+	 (let ((inhibit-quit t))
 	   ,@body
 	   ;; XEmacs does not draw attachments unless sit for 0...
 	   (mew-redraw)
@@ -153,7 +152,7 @@
 	(old-in-reply-to (mew-idstr-get-first-id
 			  (mew-header-get-value mew-in-reply-to:)))
 	(old-references  (mew-idstr-to-id-list
-			  (mew-header-get-value mew-references:) nil 'rev))
+			  (mew-header-get-value mew-references:) 'rev))
 	id-list in-reply-to references)
     ;; Assuming that In-Reply-To: contains one ID.
     (when old-message-id
