@@ -20,17 +20,20 @@
 ;;; Switching to folder
 ;;;
 
-(defun mew-summary-switch-to-folder (folder &optional thread)
+(defun mew-summary-switch-to-folder (folder &optional thread func)
   (let ((ofolder (mew-summary-folder-name 'ext))
 	case-folder new-folder)
     (cond
      ((get-buffer folder)
       (switch-to-buffer folder)
+      (mew-set-buffer-cs mew-cs-m17n)
       (unless (mew-folder-virtualp folder) (mew-summary-folder-cache-load))
       (unless (string= ofolder folder) (mew-window-configure 'summary)))
      (t
       (setq new-folder t)
-      (switch-to-buffer (get-buffer-create folder))
+      (unless func (setq func 'switch-to-buffer))
+      (funcall func (get-buffer-create folder))
+      (mew-set-buffer-cs mew-cs-m17n)
       (mew-buffers-setup folder)
       (if (null (setq case-folder (mew-folder-case folder)))
 	  (mew-sinfo-set-folder folder)
@@ -185,6 +188,7 @@ the message is then displayed."
 If 'mew-summary-jump-top-then-display' is non-nil, 
 the top message is then displayed."
   (interactive)
+  (if mew-use-push-mark (mew-push-mark))
   (goto-char (point-min))
   (if mew-summary-jump-top-then-display
       (mew-summary-display nil)))
@@ -194,6 +198,7 @@ the top message is then displayed."
 If 'mew-summary-jump-bottom-then-display' is non-nil, 
 the top message is then displayed."
   (interactive)
+  (if mew-use-push-mark (mew-push-mark))
   (goto-char (point-max))
   (unless (bobp) (forward-line -1))
   (if mew-summary-jump-bottom-then-display
@@ -278,12 +283,13 @@ the top message is then displayed."
 (defun mew-summary-pipe-message (prefix command)
   "Send the content of Message buffer to a command via pipe.
 If called with '\\[universal-argument]', the body of the message
-(excluding its header) is sent."
+\(excluding its header) is sent."
   (interactive
    (list current-prefix-arg 
 	 (read-string "Shell command on message: " mew-last-shell-command)))
   (mew-summary-display 'redisplay)
-  (when (y-or-n-p "Send this message to pipe? ")
+  (when (or (not mew-ask-pipe)
+            (y-or-n-p "Send this message to pipe? "))
     (save-excursion
       (set-buffer (mew-buffer-message))
       (save-restriction
@@ -572,39 +578,38 @@ Sometime a process accidentally remains in Summary mode.
 In this situation, you cannot execute '\\[mew-summary-retrieve]', '\\[mew-summary-ls]', nor '\\[mew-summary-exec]'.
 Use this command to solve this problem.
 
-If called with '\\[universal-argument]', corresponding SSH process, if
-any, is also killed. Please note that the SSH process is shared by 
-other network processes. In this situation, if you kill the SSH process,
-the other network processes are also hung up."
+If called with '\\[universal-argument]', all SSH processes, if
+any, are killed."
   (interactive "P")
-  (unwind-protect
-      (let ((process mew-summary-buffer-process)
-	    (disp-msg (interactive-p))
-	    pnm ssh-process)
-	(if (not (processp process))
-	    (if disp-msg (message "No process to kill. This buffer is unlocked anyway"))
-	  (cond
-	   ((eq process t)
-	    ;; Emacs is locking
-	    )
-	   ((memq (process-status process) '(open closed))
-	    ;; Network process
-	    (setq pnm (process-name process))
-	    (setq ssh-process (mew-net-get-ssh-process pnm))
-	    (mew-info-clean-up pnm)
-	    (delete-process process)
-	    (mew-summary-visible-buffer (current-buffer))
-	    (when (mew-sinfo-get-refile-back)
-	      (mew-sinfo-set-refile
-	       (nconc (mew-sinfo-get-refile) (mew-sinfo-get-refile-back)))
-	      (mew-sinfo-set-refile-back nil))
-	    (if (and kill-ssh ssh-process) (kill-process ssh-process))
-	    (mew-summary-unlock))
-	   (t
-	    ;; Local process or SSL
-	    (kill-process process)))
-	  (if disp-msg (message "The process was killed"))))
-    (mew-summary-unlock)))
+  (let ((process mew-summary-buffer-process)
+	(disp-msg (interactive-p))
+	pros sentinel)
+    (if kill-ssh
+	(progn
+	  (setq pros (process-list))
+	  (while pros
+	    (if (string-match "mew" (process-name (car pros)))
+		(delete-process (car pros)))
+	    (setq pros (cdr pros)))
+	  (if disp-msg (message "All SSH processes were killed")))
+      (cond
+       ((not (processp process))
+	(if disp-msg (message "No process to kill. This buffer is unlocked anyway")))
+       ((eq process t)
+	;; Emacs is locking
+	(if disp-msg (message "This buffer was unlocked")))
+       ((memq (process-status process) '(open closed))
+	;; Network process
+	(setq sentinel (process-sentinel process))
+	(if (fboundp sentinel)
+	    (funcall sentinel process "killed by the user"))
+	(delete-process process)
+	(if disp-msg (message "The process was killed")))
+       (t
+	;; Local process or SSL
+	(kill-process process)
+	(if disp-msg (message "The process was killed"))))
+      (mew-summary-unlock))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -641,7 +646,7 @@ the other network processes are also hung up."
 	  mew-cs-text-for-read mew-cs-text-for-write
 	  (while files
 	    (mew-erase-buffer)
-	    (insert-file (car files))
+	    (mew-insert-file-contents (car files))
 	    (goto-char (point-min))
 	    (if (re-search-forward mew-eoh nil t)
 		(forward-line))
@@ -745,7 +750,7 @@ message."   (interactive)
 	   (setq idx (1- (string-to-int idx)))
 	   (if (and (>= idx 0) (< idx len))
 	       (aset vec idx num)
-	     (error "illegal number")))
+	     (error "invalid number")))
 	 (unless TTL (error "no total"))
 	 (setq i 0)
 	 (while (< i len)
@@ -759,7 +764,7 @@ message."   (interactive)
 	  (goto-char (point-max))
 	  (save-restriction
 	    (narrow-to-region (point) (point))
-	    (insert-file-contents (mew-expand-folder cfld (aref vec 0)))
+	    (mew-insert-file-contents (mew-expand-folder cfld (aref vec 0)))
 	    ;; Removing unnecessary fields from the encapsulating
 	    ;; (outer) header.
 	    (goto-char (point-min))
@@ -792,7 +797,7 @@ message."   (interactive)
 	    (goto-char (point-max))
 	    (save-restriction
 	      (narrow-to-region (point) (point))
-	      (insert-file-contents (mew-expand-folder cfld (aref vec i)))
+	      (mew-insert-file-contents (mew-expand-folder cfld (aref vec i)))
 	      (goto-char (point-min))
 	      (re-search-forward mew-eoh nil t)
 	      (forward-line)
@@ -803,32 +808,44 @@ message."   (interactive)
 			nil 'no-msg)))
        (mew-touch-folder folder)
        (message "Joining...done")
-       (if (y-or-n-p (format "Go to %s? " folder))
-	   (mew-summary-visit-folder folder 'goend)))))))
+       (when mew-ask-folder-after-join
+	 (if (y-or-n-p (format "Go to %s? " folder))
+	     (mew-summary-visit-folder folder 'goend))))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Cleaning +trash
 ;;;
 
-(defun mew-summary-clean-trash (&optional arg)
-  "Remove all messages in +trash.
-When called with '\\[universal-argument]', you can specify a target folder
-to clean up."
-  (interactive "P")
-  (let* ((folder mew-trash-folder)
-	 trashdir msgs)
-    (if arg (setq folder (mew-input-folder nil mew-trash-folder)))
-    (when folder
-      (setq trashdir (mew-expand-folder folder))
-      (setq msgs (mew-dir-messages trashdir))
-      (if (null msgs)
-	  (message "No messages to be removed in %s" folder)
-	(when (y-or-n-p (format "Remove all messages in %s? " folder))
-	  (message "Removing all messages in %s..." folder)
-	  (mew-summary-unlink-msgs folder msgs)
-	  (mew-summary-folder-cache-clean folder)
-	  (message "Removing all messages in %s...done" folder))))))
+(defun mew-summary-clean-trash ()
+  "Remove all messages in +trash."
+  (interactive)
+  (let* ((case (mew-sinfo-get-case))
+	 (folder (mew-sinfo-get-folder))
+	 case-trash trash trashdir msgs)
+    (cond
+     ((mew-folder-imapp folder)
+      (setq trash (mew-imap-trash-folder case)))
+     (t
+      (setq trash mew-trash-folder)))
+    (setq case-trash (mew-input-folder case trash))
+    (setq case (car (mew-folder-case case-trash))) ;; xxx
+    (setq trash (mew-folder-folder case-trash))
+    (setq trashdir (mew-expand-folder trash))
+    (setq msgs (mew-dir-messages trashdir))
+    (if (and (mew-folder-localp trash) (null msgs))
+	(message "No messages to be removed in %s" trash)
+      (if (not (y-or-n-p (format "Remove all messages in %s? " case-trash)))
+	  (message "No messages removed in %s" trash)
+	(message "Removing all messages in %s..." case-trash)
+	(mew-summary-unlink-msgs case-trash msgs)
+	(mew-summary-folder-cache-clean case-trash)
+	(message "Removing all messages in %s...done" case-trash)
+	(when (mew-folder-imapp trash)
+	  (if (get-buffer case-trash)
+	      (set-buffer case-trash)
+	    (mew-summary-switch-to-folder case-trash nil 'set-buffer))
+	  (mew-imap-retrieve case 'exec case-trash '(("1:*") nil nil nil 0 -1)))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -909,11 +926,105 @@ folder."
   (with-temp-buffer
     (mew-frwlet
      mew-cs-text-for-read mew-cs-text-for-write
-     (insert-file-contents srcfile)
+     (mew-insert-file-contents srcfile)
      (goto-char (point-min))
      (mew-header-delete-lines (list mew-x-mew-uidl:))
      (write-region (point-min) (point-max) dstfile nil 'no-msg))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Deleting and moving a folder
+;;;
+
+(defun mew-summary-delete-folder ()
+  "Delete this folder."
+  (interactive)
+  (mew-summary-only
+   (let* ((folder (or (mew-sinfo-get-folder)
+		      (mew-summary-folder-name 'ext)))
+	  (case (mew-sinfo-get-case))
+	  (case:folder (mew-summary-folder-name 'ext))
+	  (dir (mew-expand-folder case:folder))
+	  (buf (current-buffer)))
+     (cond
+      ((mew-folder-localp folder)
+       (if (mew-folder-node-p folder)
+	   (message "%s cannot be deleted" folder)
+	 (when (y-or-n-p "Delete this folder? ")
+	   (mew-delete-directory-recursively dir)
+	   (mew-local-folder-delete folder)
+	   (mew-summary-goto-folder mew-inbox-folder)
+	   (mew-kill-buffer buf))))
+      ((mew-folder-imapp folder)
+       (if (mew-folder-node-p folder case)
+	   (message "%s cannot be deleted" folder)
+	 (when (y-or-n-p "Delete this folder? ")
+	   (setq mew-summary-buffer-process-error nil)
+	   (mew-imap-retrieve case 'delete case:folder)
+	   (mew-timing)
+	   (mew-rendezvous mew-summary-buffer-process)
+	   (if mew-summary-buffer-process-error
+	       (progn
+		 (message "%s cannot be deleted" folder)
+		 (setq mew-summary-buffer-process-error nil))
+	     (mew-imap-folder-delete case folder)
+	     (mew-delete-directory-recursively dir) ;; cache
+	     (mew-summary-goto-folder mew-imap-inbox-folder)
+	     (mew-kill-buffer buf)))))))))
+
+(defun mew-summary-rename-folder ()
+  "Rename this folder."
+  (interactive)
+  (mew-summary-only
+   (let* ((folder (or (mew-sinfo-get-folder)
+		      (mew-summary-folder-name 'ext)))
+	  (case (mew-sinfo-get-case))
+	  (case:folder (mew-summary-folder-name 'ext))
+	  (dir (mew-expand-folder case:folder))
+	  (buf (current-buffer))
+	  new-folder new-dir case:new-folder)
+     (cond
+      ((mew-folder-localp folder)
+       (setq new-folder (mew-input-local-folder folder))
+       (cond
+	((string= folder new-folder)
+	 (message "%s was not moved" folder))
+	(t
+	 (setq new-dir (mew-expand-folder new-folder))
+	 ;; xxx if new-dir exists?
+	 (rename-file dir new-dir)
+	 (mew-summary-switch-to-folder new-folder)
+	 (mew-kill-buffer buf)
+	 (if (mew-folder-node-p folder)
+	     ;; xxx remove all related buffer?
+	     (mew-local-update 'force)
+	   (mew-local-folder-delete folder)
+	   (mew-local-folder-insert new-folder)))))
+      ((mew-folder-imapp folder)
+       (setq new-folder (car (mew-input-refile-folders (list folder) t case mew-folder-imap)))
+       (setq case:new-folder (mew-folder-case-folder case new-folder))
+       (cond
+	((string= folder new-folder)
+	 (message "%s was not moved" folder))
+	(t
+	 (setq new-dir (mew-expand-folder new-folder))
+	 (mew-imap-retrieve case 'rename case:folder case:new-folder)
+	 (mew-timing)
+	 (mew-rendezvous mew-summary-buffer-process)
+	 (if mew-summary-buffer-process-error
+	     (progn
+	       (message "%s cannot be renamed" folder)
+	       (setq mew-summary-buffer-process-error nil))
+	   (rename-file dir new-dir t) ;; cache
+	   (mew-summary-switch-to-folder case:new-folder)
+	   (mew-kill-buffer buf)
+	   (if (mew-folder-node-p folder case)
+	       (if (y-or-n-p "Collect a folder list now? ")
+		   (mew-imap-update case)
+		 (message (substitute-command-keys "Type '3\\<mew-summary-mode-map>\\[mew-status-update]' to collect a folder list.")))
+	     (mew-imap-folder-delete case folder)
+	     (mew-imap-folder-insert case new-folder))))))))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Toggle
@@ -1015,6 +1126,59 @@ If 'mew-warning-field-level' is 1, set it to 2."
       (message "'mew-debug' is now 't'")
     (message "'mew-debug' is now 'nil'")))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Learning spam and ham
+;;;
+
+(defun mew-summary-learn-spam ()
+  "Learn that this message is a spam.
+See also 'mew-spam-prog' and 'mew-spam-prog-args."
+  (interactive)
+  (mew-summary-msg
+   (let* ((fld (mew-summary-folder-name))
+	  (msg (mew-summary-message-number))
+	  (file (mew-expand-folder fld msg))
+	  (case (mew-sinfo-get-case))
+	  (prog (mew-spam-prog case))
+	  (args (mew-spam-prog-args case))
+	  beg end)
+     (message "Learning as spam...")
+     (when (mew-which-exec prog)
+       (with-temp-buffer
+	 (apply 'call-process prog file (current-buffer) nil args)
+	 (setq beg (point-min))
+	 (goto-char beg)
+	 (forward-line)
+	 (if (= (point) beg)
+	     (message "Learning as spam...done")
+	   (setq end (1- (point)))
+	   (message (concat "Learned as spam: " (mew-buffer-substring beg end)))))))))
+
+(defun mew-summary-learn-ham ()
+  "Learn that this message is a ham.
+See also 'mew-ham-prog' and 'mew-ham-prog-args."
+  (interactive)
+  (mew-summary-msg
+   (let* ((fld (mew-summary-folder-name))
+	  (msg (mew-summary-message-number))
+	  (file (mew-expand-folder fld msg))
+	  (case (mew-sinfo-get-case))
+	  (prog (mew-ham-prog case))
+	  (args (mew-ham-prog-args case))
+	  beg end)
+     (message "Learning as ham...")
+     (when (mew-which-exec prog)
+       (with-temp-buffer
+	 (apply 'call-process prog file (current-buffer) nil args)
+	 (setq beg (point-min))
+	 (goto-char beg)
+	 (forward-line)
+	 (if (= (point) beg)
+	     (message "Learning as ham...done")
+	   (setq end (1- (point)))
+	   (message (concat "Learned as ham: " (mew-buffer-substring beg end)))))))))
+
 (provide 'mew-summary4)
 
 ;;; Copyright Notice:
