@@ -87,8 +87,8 @@ cdr is new text.")
 
 (defvar mew-error-unknown-charset    "**UNKNOWN CHARSET**")
 (defvar mew-error-broken-string      "**BROKEN STRING**")
-(defvar mew-error-illegal-b-encoding " **B ENCODING ERROR** ")
-(defvar mew-error-illegal-q-encoding " **Q ENCODING ERROR** ")
+(defvar mew-error-invalid-b-encoding " **B ENCODING ERROR** ")
+(defvar mew-error-invalid-q-encoding " **Q ENCODING ERROR** ")
 (defvar mew-error-no-subject         "** no subject **")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -190,19 +190,19 @@ for privacy reasons.")
 ;;;
 
 (defsubst mew-ct-textp (ct)
-  (string-match "^Text/" (capitalize ct)))
+  (string-match "^Text" (capitalize ct)))
 
 (defsubst mew-ct-imagep (ct)
-  (string-match "^Image/" (capitalize ct)))
+  (string-match "^Image" (capitalize ct)))
 
 (defsubst mew-ct-modelp (ct)
-  (string-match "^Model/" (capitalize ct)))
+  (string-match "^Model" (capitalize ct)))
 
 (defsubst mew-ct-multipartp (ct)
-  (string-match "^Multipart/" (capitalize ct)))
+  (string-match "^Multipart" (capitalize ct)))
 
 (defsubst mew-ct-messagep (ct)
-  (string-match "^Message/" (capitalize ct)))
+  (string-match "^Message" (capitalize ct)))
 
 (defsubst mew-ct-linebasep (ct)
   (or (mew-ct-textp ct)
@@ -281,7 +281,7 @@ in draft buffer.")
   "*The default Content-Type: for a file whose suffix is unknown.")
 
 (defvar mew-mime-content-type
-  '(("multipart/"  nil         nil     nil            mew-icon-multipart)
+  '(("multipart"    nil         nil     nil            mew-icon-multipart)
     ;;
     ("audio/basic"  "\\.au$"    mew-b64 mew-prog-audio mew-icon-audio)
     ("audio/x-wav"  "\\.wav$"   mew-b64 mew-prog-audio mew-icon-audio)
@@ -299,12 +299,12 @@ in draft buffer.")
     ("image/x-bmp" "\\.bmp$"   mew-b64 mew-prog-image mew-icon-image)
     ("image/x-pcx" "\\.pcx$"   mew-b64 mew-prog-image mew-icon-image)
     ("image/x-tga" "\\.tga$"   mew-b64 mew-prog-image mew-icon-image)
-    ("image/"      "^$"        mew-b64 mew-prog-image mew-icon-image)
+    ("image"       "^$"        mew-b64 mew-prog-image mew-icon-image)
     ;;
     ("model/iges" "\\.ige?s$" mew-b64 mew-prog-iges  mew-icon-image) ;; xxx
     ("model/vrml" "\\.wrl$"   mew-b64 mew-prog-vrml  mew-icon-image)
     ("model/mesh" "\\.me?sh$" mew-b64 mew-prog-mesh  mew-icon-image)
-    ("model/"      "^$"       mew-b64 mew-prog-model mew-icon-image)
+    ("model"      "^$"        mew-b64 mew-prog-model mew-icon-image)
     ;;
     ("video/mpeg"      "\\.mpe?g$" mew-b64 mew-prog-video mew-icon-video)
     ("video/quicktime" "\\.mov$"   mew-b64 mew-prog-video mew-icon-video)
@@ -325,7 +325,7 @@ in draft buffer.")
      mew-prog-xml2                   mew-icon-text)
     ("application/msword"            "\\.doc$"  mew-b64
      mew-prog-msword                 mew-icon-text)
-    ("application/vnd.ms-excel"      "\\.xls$"  mew-b64
+    ("application/vnd.ms-excel"      "\\.xl[st]$" mew-b64
      mew-prog-msexcel                mew-icon-text)
     ("application/vnd.ms-powerpoint" "\\.ppt$"  mew-b64
      mew-prog-mspowerpoint           mew-icon-text)
@@ -352,7 +352,7 @@ in draft buffer.")
     ("text/xml"      "\\.xml$"   nil     mew-prog-xml       mew-icon-text)
     ("text/rfc822-headers" "\\.hdr$" nil
      mew-prog-rfc822-headers     mew-icon-message/rfc822)
-    ("text/"         "^$"        nil     mew-prog-text      mew-icon-text)
+    ("text"         "^$"        nil     mew-prog-text      mew-icon-text)
     ;; Unkown CT: matches here.
     (t               "^$"       nil   mew-prog-octet-stream mew-icon-unknown)
     ;; Unknown suffix matches here and return the entry specified 
@@ -441,11 +441,11 @@ the next version of Mew.")
 ;;;
 
 (defvar mew-mime-content-disposition
-  '(("text/"      "inline"     t)
-    ("image/"     "inline"     t)
-    ("message/"   "inline"     nil)
-    ("multipart/" nil          nil)
-    (t            "attachment" t))
+  '(("text"      "inline"     t)
+    ("image"     "inline"     t)
+    ("message"   "inline"     nil)
+    ("multipart" nil          nil)
+    (t           "attachment" t))
   "(content-type inline/attachment filename)")
 
 (defsubst mew-cdpdb-by-ct (ct)
@@ -695,7 +695,7 @@ the next version of Mew.")
     ("^Face:$" nil
      mew-face-header-private
      mew-face-header-marginal)
-    ("^X-" nil
+    ("^\\(X\\|Original\\)-" nil
      mew-face-header-private
      mew-face-header-marginal))
   "*An alist of field spec for Message mode. Each spec
@@ -853,6 +853,69 @@ undo-func, exec-func, and sanity-fucn.")
 (defsubst mew-markdb-func-sanity (mark)
   (nth 7 (mew-markdb-by-mark mark)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Inbox action
+;;;
+
+(defvar mew-inbox-action-alist nil
+  "*This variable controls actions when retrieving messages.
+Currently this works only when '\\<mew-summary-mode-map>\\[mew-summary-retrieve]' with POP or 'mewl/incm'.
+You can put any marks, typically 'o' and 'D' according to message
+headers.
+
+This value must be a list of entries. An entry is a list whose first
+element is a field defined in 'mew-scan-fields'. 
+
+If the second element is a list, the first element of the list must be
+a mark (e.g. ?D) or a string which is a comma separated folders (e.g
+\"+foo,+bar\" for ?o). Other elements of the list are regular
+expressions to be matched to the value of the field.
+
+With the following configuration, Mew marks messages whose Subject:
+matches \"adult\" or \"money\" with 'D'. And Mew marks messages whose
+Subject is \"daily log\" with 'o' to be refiled \"+log\". Also Mew
+marks messages whose Content-Type: contains \"gb2312\" with 'D'.
+
+(setq mew-inbox-action-alist
+      '((\"Subject:\"
+	 (?D \"adult\" \"money\")
+	 (\"+log\" \"daily log\"))
+	(\"Content-Type:\"
+	 (?D \"gb2312\"))))
+
+If the second element of the entry is a symbol, it will be called as a
+function which returns nil or a mark or a string.
+
+If you are using 'spamassassin' which adds the \"X-Spam-Flag:\" field
+to messages and puts them in your mailbox, configure as follows:
+
+(setq mew-spam: \"X-Spam-Flag:\") ;; must be in \".emacs\"
+
+(defun mew-spam-assassin (val)
+  (let ((case-fold-search t))
+    (if (string-match \"yes\" val) ?D)))
+
+(setq mew-inbox-action-alist
+      '((\"X-Spam-Flag:\" mew-spam-assassin)))
+
+With this configuration, messages with the \"X-Spam-Flag:\" whose
+value is \"Yes\" are automatically marked with 'D' since the
+'mew-spam-assassin' function returns 'D' marks when the value is
+\"Yes\".
+
+If you are using 'bogofilter' which adds the \"X-Bogosity:\" field
+to messages and puts them to your mailbox, configure as follows:
+
+(setq mew-spam: \"X-Bogosity:\") ;; must be in \".emacs\" but default
+
+(defun mew-spam-bogofilter (val)
+  (let ((case-fold-search t))
+    (if (string-match \"yes\" val) ?D)))
+
+(setq mew-inbox-action-alist
+      '((\"X-Bogosity:\" mew-spam-bogofilter)))
+")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -860,8 +923,8 @@ undo-func, exec-func, and sanity-fucn.")
 ;;;
 
 (defvar mew-ext-url-alist
-  '(("^application/" "Fetch by w3" mew-ext-url-fetch-by-w3 nil)
-    (t		     "Browse by mozilla" "mozilla" ("-install")))
+  '(("^application" "Fetch by w3" mew-ext-url-fetch-by-w3 nil)
+    (t		    "Browse by mozilla" "mozilla" ("-install")))
   "*Alist of (REGEXP DOC PROGRAM ARGS-LIST) to define
 an appropriate method for a content-type in external-body. Note
 this phantom body is defined RFC 2017.
@@ -873,12 +936,10 @@ If PROGRAM is a symbol, the lisp function whose name is PROGRAM is called.
 'mew-ext-url-show-by-w3' and 'mew-ext-url-fetch-by-w3' are pre-defined
 as lisp function.
 
-If you want to use \"w3.el\" instead of \"mozilla\", put the 
-following in .emacs.
+If you want to use \"w3.el\" instead of \"mozilla\", configure like this:
 	(setq mew-ext-url-alist
               '((t (\"Browse by w3\" mew-ext-url-show-by-w3 nil))))
-If you want to use \"lynx\" instead of \"mozilla\", put the 
-following in .emacs.
+If you want to use \"lynx\" instead of \"mozilla\", configure like this:
         (setq mew-ext-url-alist
               '((t (\"Browse by lynx\" \"kterm\" (\"-e\" \"lynx\" \"-color\")))))
 ")
@@ -977,27 +1038,25 @@ following in .emacs.
 
 Currently, the following keys are supported: 
 \"name\", \"user\", \"mail-domain\", 
-\"cc\", \"fcc\", \"dcc\", \"reply-to\",
-\"organization\", \"header-alist\", 
-\"smtp-server\", \"smtp-port\", \"smtp-ssh-server\", 
-\"smtp-ssl\", \"smtp-ssl-port\", 
-\"smtp-msgid-user\", \"smtp-msgid-domain\", 
-\"smtp-helo-domain\", \"smtp-mail-from\", 
+\"cc\", \"fcc\", \"dcc\", \"reply-to\", \"organization\", \"header-alist\", 
+\"proto\", 
+\"smtp-server\", \"smtp-port\", \"smtp-ssh-server\", \"smtp-ssl\", \"smtp-ssl-port\", 
 \"smtp-user\", \"smtp-auth\", \"smtp-auth-list\", 
-\"pop-server\", \"pop-port\", \"pop-ssh-server\", 
-\"pop-ssl\", \"pop-ssl-port\", 
-\"pop-user\", \"pop-auth\", 
-\"pop-size\", \"pop-body-lines\", \"pop-delete\", \"pop-folder\", 
-\"imap-server\", \"imap-port\", \"imap-ssh-server\", 
-\"imap-ssl\", \"imap-ssl-port\", 
-\"imap-user\", \"imap-auth\", \"imap-size\", \"imap-folder\", 
-\"nntp-server\", \"nntp-port\", \"nntp-ssh-server\", 
-\"nntp-ssl\", \"nntp-ssl-port\", 
-\"nntp-user\", \"nntp-size\", \"nntp-folder\",
+\"smtp-msgid-user\", \"smtp-msgid-domain\", \"smtp-helo-domain\", \"smtp-mail-from\", 
+\"pop-server\", \"pop-port\", \"pop-ssh-server\", \"pop-ssl\", \"pop-ssl-port\", 
+\"pop-user\", \"pop-auth\", \"pop-auth-list\",
+\"pop-size\", \"pop-body-lines\", \"pop-delete\", \"pop-body-lines\",
+\"imap-server\", \"imap-port\", \"imap-ssh-server\", \"imap-ssl\", \"imap-ssl-port\", 
+\"imap-user\", \"imap-auth\", \"imap-auth-list\", 
+\"imap-size\", \"imap-header-only\", \"imap-delete\", 
+\"imap-trash-folder\", \"imap-queue-folder\", \"imap-spam-field\", \"imap-spam-word\",
+\"nntp-server\", \"nntp-port\", \"nntp-ssh-server\", \"nntp-ssl\", \"nntp-ssl-port\", 
+\"nntp-user\", \"nntp-size\", \"nntp-header-only\", 
 \"nntp-msgid-user\", \"nntp-msgid-domain\", 
-\"inbox-folder\", \"queue-folder\",
+\"inbox-folder\", \"queue-folder\", \"postq-folder\", 
 \"mailbox-type\", \"mbox-command\", \"mbox-command-arg\", 
-\"signature-file\", \"content-type\" .
+\"use-retrieve\", \"signature-file\", \"content-type\", \"refile-guess-alist\",
+\"spam-prog\", \"spam-prog-args\", \"ham-prog\", \"ham-prog-args\".
 
 from = name <user@mail-domain>
 message-id = *random*.smtp-msgid-user@smtp-msgid-domain
