@@ -76,14 +76,14 @@ execute 'mew-summary-virtual-with-external'. Otherwise,
 (defun mew-virtual-folder-name ()
   (if (mew-virtual-p)
       (car (mew-vinfo-get-flds))
-    (mew-summary-folder-name)))
+    (mew-summary-folder-name 'ext)))
 
 (defun mew-summary-virtual-with-internal ()
   "Create Virtual mode with mewl."
   (let* ((flds (mew-input-folders (mew-virtual-folder-name)))
 	 folders vfolder pattern opts lra)
     (when flds
-      (setq folders (mapcar 'mew-folder-remote-to-local flds))
+      (setq folders (mapcar 'mew-expand-folder2 flds))
       (setq pattern (mew-input-pick-pattern))
       (setq vfolder (mew-summary-get-vfolder flds))
       (mew-summary-switch-to-folder vfolder)
@@ -108,7 +108,7 @@ This function MUST returns a file name which contains message numbers.")
   (let* ((flds (mew-input-folders (mew-virtual-folder-name)))
 	 folders vfolder pattern file func opts lra)
     (when flds
-      (setq folders (mapcar 'mew-folder-remote-to-local flds))
+      (setq folders (mapcar 'mew-expand-folder2 flds))
       (setq pattern (mew-input-grep-pattern))
       (mew-sinfo-set-find-key pattern) ;; xxx necessary?
       (setq vfolder (mew-summary-get-vfolder flds))
@@ -185,14 +185,14 @@ This function MUST returns a file name which contains message numbers.")
 ;;;
 
 (defun mew-summary-mark-virtual ()
-  "Making Virtaul mode for messages marked with '*'."
+  "Making Virtual mode for messages marked with '*'."
   (interactive)
   (if (not (mew-summary-p))
       (message "This command cannot be used in this mode")
     (let* ((folder (mew-summary-folder-name 'ext))
 	   (vfolder (mew-summary-get-vfolder (list folder)))
 	   (start (point))
-	   beg line)
+	   beg line med)
       (mew-summary-switch-to-folder vfolder)
       (mew-erase-buffer)
       (mew-vinfo-set-flds (list folder))
@@ -207,9 +207,17 @@ This function MUST returns a file name which contains message numbers.")
 	(save-excursion
 	  (set-buffer vfolder)
 	  (mew-elet
-	   (insert line))))
+	   (insert line)
+	   (save-excursion
+	     (when (and (search-backward "\r")
+			(setq med (point))
+			(looking-at mew-regex-sumsyn-short))
+	       (goto-char (match-beginning 1))
+	       (insert folder)
+	       (put-text-property med (point) 'invisible t))))))
       (goto-char start)
       (set-buffer vfolder)
+      (mew-summary-set-count-line)
       ;; Unmarking in both Summary and Thread
       (mew-mark-undo-mark mew-mark-review 'no-msg))))
 
