@@ -35,6 +35,7 @@
 (defvar mew-w32-prog-print     "notepad.exe")
 (defvar mew-w32-prog-print-arg nil)
 
+(defvar mew-w32-cs-print nil)
 
 ;; Emacs version dependent variables.
 (cond
@@ -48,7 +49,7 @@
       (progn
 	(require 'mw32script)
 	(mw32script-init))
-    (if (require 'mw32script nil t) ;; NTEmacs
+    (if (condition-case nil (require 'mw32script) (file-error nil)) ;; NTEmacs
 	(progn
 	  (mw32script-make-pathext-regexp)
 	  (defalias 'call-process-original 'call-process)
@@ -171,7 +172,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2002 Mew developing team.
+;; Copyright (C) 1996-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
