@@ -87,8 +87,8 @@ If event is button 3, show a menu."
   (interactive)
   (let* ((fld (mew-summary-folder-name))
 	 (msg (mew-summary-message-number))
-	 (num (mew-syntax-number))
-	 (nums (mew-syntax-number-to-nums num))
+	 (part (mew-syntax-number))
+	 (nums (mew-syntax-number-to-nums part))
 	 (button (event-button last-command-event))
 	 (cache (mew-cache-hit fld msg 'must-hit)))
     (mew-summary-goto-part msg part)
@@ -151,7 +151,6 @@ If event is button 3, show a menu."
 		 (program (nth 0 attr))
 		 (options (nth 1 attr))
 		 (async   (nth 2 attr))
-		 (zmacs-regions nil) ;; for XEmacs
 		 (format (cond
 			  ((and (string-match "jpeg" ct)
 				(valid-image-instantiator-format-p 'jpeg))
@@ -176,7 +175,7 @@ If event is button 3, show a menu."
 		  (setq glyph (make-glyph (vector format :file file)))
 		  (if (eq format 'xbm)
 		      (set-glyph-property glyph 'face 'mew-bitmap))
-		  (set-extent-begin-glyph
+		  (set-extent-end-glyph
 		   (mew-overlay-make (point-max) (point-max)) glyph)
 		  (message "Loading image...done"))
 	      (if (and (stringp program) (mew-which-exec program))
@@ -213,8 +212,6 @@ popup menu can be implemented."
 ;;;
 
 (defun mew-summary-setup-decoration ()
-  (make-local-variable 'zmacs-regions)
-  (setq zmacs-regions nil)
   (if (featurep 'scrollbar)
       (set-specifier scrollbar-height (cons (current-buffer) 0)))
   (mew-summary-toolbar-update)
@@ -270,17 +267,45 @@ popup menu can be implemented."
     (setq mew-x-emacs-end-of-message
 	  (make-glyph
 	   (vector 'string :data mew-end-of-message-string))))
-  (overlay-put (mew-minfo-get-overlay) 'begin-glyph mew-x-emacs-end-of-message))
+  (overlay-put (mew-minfo-get-eom) 'end-glyph mew-x-emacs-end-of-message))
 
 (defsubst mew-message-set-end-of-part ()
   (unless (glyphp mew-x-emacs-end-of-part)
     (setq mew-x-emacs-end-of-part
 	  (make-glyph
 	   (vector 'string :data mew-end-of-part-string))))
-  (overlay-put (mew-minfo-get-overlay) 'begin-glyph mew-x-emacs-end-of-part))
+  (overlay-put (mew-minfo-get-eom) 'end-glyph mew-x-emacs-end-of-part))
 
 (defsubst mew-message-set-end-of-nil ()
-  (overlay-put (mew-minfo-get-overlay) 'begin-glyph nil))
+  (overlay-put (mew-minfo-get-eom) 'end-glyph nil))
+
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Veil
+;;;
+
+(defvar mew-x-emacs-header-veil nil)
+
+(defun mew-header-veil-make ()
+  (unless (glyphp mew-x-emacs-header-veil)
+    (setq mew-x-emacs-header-veil
+	  (make-glyph
+	   (vector 'string :data mew-header-veil-string))))
+  (let ((ov (mew-overlay-make 1 1)))
+    (overlay-put ov 'invisible t)
+    (overlay-put ov 'end-glyph mew-x-emacs-header-veil)
+    (delete-overlay ov) ;; detach from the buffer
+    ov))
+
+(defun mew-toggle-header-veil (ov)
+  (cond
+   ((overlay-get ov 'invisible)
+    (overlay-put ov 'invisible nil)
+    (overlay-put ov 'end-glyph nil))
+   (t
+    (overlay-put ov 'invisible t)
+    (overlay-put ov 'end-glyph mew-x-emacs-header-veil))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -319,14 +344,15 @@ popup menu can be implemented."
    (t
     (set-buffer (mew-buffer-message))
     (mew-elet
-     (set-extent-end-glyph (mew-overlay-make (point-min) (point-min))
-			   (make-glyph (vector 
-					format
-					:data
-					(save-excursion
-					  (set-buffer cache)
-					  (mew-buffer-substring 
-					   begin end))))))))
+     (set-extent-end-glyph
+      (mew-overlay-make (point-min) (point-min))
+      (make-glyph (vector 
+		   format
+		   :data
+		   (save-excursion
+		     (set-buffer cache)
+		     (mew-buffer-substring 
+		      begin end))))))))
   (message "Loading image...done"))
 
 ;; XEmacs depends on libX when it converts XBM to glyph.
@@ -392,16 +418,16 @@ popup menu can be implemented."
 		 (setq size1 (mew-match-string 1)))))
 	   (mew-erase-buffer)
 	   (insert image)
-	   (when (or should-unoptimize should-be-non-interlaced))
-	   ;; Unoptimize anyway.
-	   (if should-unoptimize
+	   (when (or should-unoptimize should-be-non-interlaced)
+	     ;; Unoptimize anyway.
+	     (if should-unoptimize
+		 (call-process-region (point-min) (point-max)
+				      prog
+				      t t nil should-unoptimize
+				      "--no-interlace")
 	       (call-process-region (point-min) (point-max)
 				    prog
-				    t t nil should-unoptimize
-				    "--no-interlace")
-	     (call-process-region (point-min) (point-max)
-				  prog
-				  t t nil "--no-interlace"))
+				    t t nil "--no-interlace")))
 	   (goto-char (point-min))
 	   (unless (looking-at "GIF8[79]")
 	     ;; Unoptimization is failed. :-<
@@ -430,7 +456,7 @@ popup menu can be implemented."
 	  overlay)
       (when (re-search-forward regex2 nil t)
 	(setq overlay (mew-overlay-make (match-end 1) (match-end 1)))
-	(set-extent-begin-glyph overlay xface)))))
+	(set-extent-end-glyph overlay xface)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
